$title ex_aux_01.gms: 補助変数 (auxliary variable) の利用例：内生的な一括税
display "@ ex_aux_01.gms: 補助変数 (auxliary variable) の利用例：内生的な一括税";
$ontext
Time-stamp:     <2023-05-01 15:57:56 st>
First-written:  <2010/03/12>

□ Lump-sum tax (一括税) があるケース．
・lump-sum tax の水準は外生的に与えられる政府支出の水準を実現するように決まる．
・よって，lump-sum tax は内生的に決ってくる．


□ SAM (ベンチマークデータ)
           |             Sectors            |     Consumers
Commoditiy |              (部門)            |      (消費者)
   (財)    |    x       y       u      g    |    cons     gov
------------------------------------------------------------
       px  |  100             -90     -10   |
       py  |          100     -80     -20   |
       pu  |                  170           |    -170
       pl  |  -25     -75                   |     100
       pk  |  -75     -25                   |     100
       pg  |                           30   |             -30
   ----------------------------------------------------------
     vlst  |                                |     -30      30
--------------------------------------------------------------

□ 政府
・政府 (gov) は消費者 (cons) から lump-sum tax をとっている．
・政府は税収を政府支出に支出する．
・政府支出は「政府支出財」の購入という形式で表現される．
・政府支出財の価格は pg．
・政府支出財は政府支出財部門 (g部門) によって生産される．


□ 政府支出財部門の生産関数
・以下のような一段階のCES型関数．

     g
    / \ <- sig_g
   /   \
  /     \
 x      y


□ 部門 x の生産関数

         x
        / \ <- sig_x
       /   \
      /     \
     k      l

・CES 型関数．
・投入物は資本，労働のみ．
・代替の弾力性は sig_x．

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
               x       y       u       g     cons     gov
px           100       0     -90     -10        0       0
py             0     100     -80     -20        0       0
pu             0       0     170       0     -170       0
pl           -25     -75       0       0      100       0
pk           -75     -25       0       0      100       0
pg             0       0       0      30        0     -30
vlst           0       0       0       0      -30      30
;
display sam;

parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
display px0, py0, pu0, pk0, pl0;

parameter
    x0          x 財のベンチマーク生産量
    y0          y 財のベンチマーク生産量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
;

x0 = sam("px","x") / px0;
y0 = sam("py","y") / py0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;

display x0, y0, kx0, ky0, lx0, ly0;

parameter
    vlst0       ベンチマークにおける lump-sum tax
;

vlst0 = sam("vlst","gov");
display vlst0;

parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
;
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;

display u0, cx0, cy0;

parameter
    pg0         ベンチマークにおける政府支出財の価格
    g0          ベンチマークにおける政府支出
    xg0         ベンチマークにおける x 財に対する政府支出需要
    yg0         ベンチマークにおける y 財に対する政府支出需要
;
pg0 = 1;

g0 = sam("pg","g") / pg0;
xg0 = - sam("px","g") / px0;
yg0 = - sam("py","g") / py0;

display pg0, g0, xg0, yg0;

parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    inc0        ベンチマーク所得
;
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;
inc0 = - sam("pu","cons");

display e_l, e_k, inc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

parameter
    sig_x       EOS: 部門 x における資本・労働の EOS    / 0.5 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
    sig_g       EOS: 政府支出における EOS               / 1.0 /
;
display sig_x, sig_y;

parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
    s_gov       政府支出のスケールファクター    / 1.0 /
;
display s_l, s_k, s_gov;

$ontext
MPSGEモデル

$offtext

$ontext
$model:ex_aux_01

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量
    u           ! u 部門の生産量 (効用水準)
    g           ! g 部門の生産量 (政府支出水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格
    pg          ! 政府支出財の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得
    gov         ! 政府の所得

*       補助変数の宣言
$auxiliary:
    lst         ! 一括税

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:x  s:sig_x
    o:px        q:x0   p:px0
    i:pk        q:kx0  p:pk0
    i:pl        q:lx0  p:pl0

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0

*       部門 g の生産関数 (政府支出)
$prod:g  s:sig_g
    o:pg        q:g0
    i:px        q:xg0  p:px0
    i:py        q:yg0  p:py0

*       ----------------------------------------------------------------------
*       予算制約

*       民間消費者の予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)
    e:pu        q:(-vlst0)      r:lst

*       政府の予算制約
$demand:gov
    d:pg
    e:pu        q:(vlst0)       r:lst

*       ----------------------------------------------------------------------
*       制約

*       補助変数 lst の決定条件: lst は政府支出 (g * g0) が外生的に与えられる水
*       準 (s_gov * g0) に等しくなるように決定． 
$constraint:lst
    g * g0 =e= s_gov * g0;

$offtext

$sysinclude mpsgeset ex_aux_01

*       変数の初期値
$ontext
補助変数のデフォールトの初期値がは 0．補助変数に 0 以外の初期値を設定するときは，
自分で設定する．
$offtext

*       lst には 1 を設定する．すると vlst0*lst = 30 が初期の lump-sum tax とな
*       る．
lst.l = 1;

*       結果表示用のパラメータ
parameter
    chk_results
;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data が 均衡となっているかをチェックする．
$offtext

ex_aux_01.iterlim = 0;
$include ex_aux_01.gen
solve ex_aux_01 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

ex_aux_01.iterlim = 10000;
$include ex_aux_01.gen
solve ex_aux_01 using mcp;

*       "b" はベンチマークの値を表す．
chk_results("x","b") = x.l;
chk_results("y","b") = y.l;
chk_results("u","b") = u.l;
chk_results("px","b") = px.l / pu.l;
chk_results("py","b") = py.l / pu.l;
chk_results("pk","b") = pk.l / pu.l;
chk_results("pl","b") = pl.l / pu.l;

*       ----------------------------------------------------------------------
*       政府支出を 2 倍に増加する．
display "@ 政府支出を 2 倍に増加する．";
$ontext

Q:
・変数 g はどう変化するか？
・一括税はどう変化するか？
・消費者の効用はどう変化するか？
$offtext

s_gov = 2;

$include ex_aux_01.gen
solve ex_aux_01 using mcp;

chk_results("x","cf") = x.l;
chk_results("y","cf") = y.l;
chk_results("u","cf") = u.l;
chk_results("px","cf") = px.l / pu.l;
chk_results("py","cf") = py.l / pu.l;
chk_results("pk","cf") = pk.l / pu.l;
chk_results("pl","cf") = pl.l / pu.l;

display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 80
* coding: utf-8
* End:
