$title ex_carbon.gms: サンプルのプログラム．
display "@ ex_carbon.gms: サンプルのプログラム．";
$ontext
Time-stamp:     <2023-05-01 16:26:50 st>
First-written:  <2010/03/12>

□ 説明
・このプログラムには他のサンプルの要素をまとめていれてある．
・他のプログラムより説明を詳しくしている．

□ SAM (ベンチマークデータ)
           |             Sectors                    |     Consumers
Commoditiy |              (部門)                    |      (消費者)
   (財)    |    x       y       z       u       g   |    cons     gov
-----------------------------------------------------------------------
       px  |  150     -10       0    -120     -20   |       0       0
       py  |  -15     130       0     -95     -20   |       0       0
       pz  |  -10     -20      45     -15       0   |       0       0
       pu  |    0       0       0     230       0   |    -230       0
       pl  |  -40     -75      -5       0       0   |     120       0
       pk  |  -60     -20     -10       0       0   |      90       0
       pr  |    0       0     -30       0       0   |      30       0
       pg  |    0       0       0       0      40   |       0     -40
     ----------------------------------------------------------------
   vtlump  |    0       0       0       0       0   |     -10      10
      vtl  |  -15      -5       0       0       0   |       0      20
      vtq  |  -10       0       0       0       0   |       0      10
-----------------------------------------------------------------------

・z財 → 化石燃料とみなす → その投入、消費からCO2が排出される。

□ 生産関数

[x 部門]
     x
     |
    / \ <- 0
   /   \
  /     \
 y     / \ <- sig_x_1
      /   \
     /     \
    l     / \ <- sig_x_2
         /   \
        /     \
       k       z

[y 部門]
     y
     |
    / \ <- 0
   /   \
  /     \
 x     / \ <- sig_y_1
      /   \
     /     \
    l     / \ <- sig_y_2
         /   \
        /     \
       k       z

[z 部門]
     z
    / \ <- sig_z_1
   /   \
  /     \
 r     / \ <- sig_z_2
      /   \
     /     \
     k      l

[u 部門]
     u
    /|\ <- sig_u
   / | \
  /  |  \
 x   y   z

□ 政府支出財部門の生産関数

     g
    / \ <- sig_g
   /   \
  /     \
 x       y

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
               x       y       z        u       g     cons     gov
px           150     -10       0     -120     -20        0       0
py           -15     130       0      -95     -20        0       0
pz           -10     -20      45      -15       0        0       0
pu             0       0       0      230       0     -230       0
pl           -40     -75      -5        0       0      120       0
pk           -60     -20     -10        0       0       90       0
pr             0       0     -30        0       0       30       0
pg             0       0       0        0      40        0     -40
vtlump         0       0       0        0       0      -10      10
vtl          -15      -5       0        0       0        0      20
vtq          -10       0       0        0       0        0      10
;
display      sam;

set i   財と部門のインデックス / x, y, z /
;
alias(i,j);

$ontext
ベンチマークにおける価格．

・ベンチマークにおける財・要素の市場価格は全て1と仮定する．
・仮に 1 以外の値を仮定したとしても，結果 (変数の変化率) は変わらな
　い．
・市場価格は 1 だが，agent 価格は 1 とは限らない．
$offtext
parameter
    p0(i)       i 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
    pr0         資源のベンチマーク市場価格
;
p0(i) = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
pr0 = 1;
display p0, pu0, pk0, pl0, pr0;

*       ----------------------------------------------------------------------
*       生産サイド

$ontext
ベンチマークの数量．

・SAM の金額データとベンチマーク価格より，ベンチマークの数量が導出できる．
$offtext
parameter
    q0(i)       i 部門の i 財のベンチマーク生産量
    int0(j,i)   i 部門の j 財のベンチマーク投入量
    kd0(i)      i 部門における資本のベンチマーク投入量
    ld0(i)      i 部門における労動のベンチマーク投入量
    rd0(i)      i 部門における資源のベンチマーク投入量
;

*       数量 = 金額 / 価格
q0("x") = sam("px","x") / p0("x");
q0("y") = sam("py","y") / p0("y");
q0("z") = sam("pz","z") / p0("z");

int0("x","x") = - sam("px","x") / p0("x");
int0("y","x") = - sam("py","x") / p0("y");
int0("z","x") = - sam("pz","x") / p0("z");
int0("x","y") = - sam("px","y") / p0("x");
int0("y","y") = - sam("py","y") / p0("y");
int0("z","y") = - sam("pz","y") / p0("z");
int0("x","z") = - sam("px","z") / p0("x");
int0("y","z") = - sam("py","z") / p0("y");
int0("z","z") = - sam("pz","z") / p0("z");
int0(i,i) = 0;

kd0("x") = - sam("pk","x") / pk0;
kd0("y") = - sam("pk","y") / pk0;
kd0("z") = - sam("pk","z") / pk0;

ld0("x") = - sam("pl","x") / pl0;
ld0("y") = - sam("pl","y") / pl0;
ld0("z") = - sam("pl","z") / pl0;

rd0("x") = - sam("pr","x") / pr0;
rd0("y") = - sam("pr","y") / pr0;
rd0("z") = - sam("pr","z") / pr0;

display q0, int0, kd0, ld0, rd0;

*       ----------------------------------------------------------------------
*       消費サイド．

parameter
    u0          ベンチマークの効用水準
    cd0(i)      財のベンチマーク消費量
;
*       ベンチマークの効用，消費量
u0 = sam("pu","u") / pu0;
cd0("x") = - sam("px","u") / p0("x");
cd0("y") = - sam("py","u") / p0("y");
cd0("z") = - sam("pz","u") / p0("z");

display u0, cd0;

*       ----------------------------------------------------------------------
*       政府

parameter
    pg0         ベンチマークにおける政府支出財の価格
    g0          ベンチマークにおける政府支出
    gd0(i)      ベンチマークにおける i 財に対する政府支出需要
;
*       これについてもベンチマーク価格は 1 と仮定．
pg0 = 1;

*       ベンチマークの生産量，投入量
g0 = sam("pg","g") / pg0;
gd0("x") = - sam("px","g") / p0("x");
gd0("y") = - sam("py","g") / p0("y");
gd0("z") = - sam("pz","g") / p0("z");
display pg0, g0, gd0;

*       ----------------------------------------------------------------------
*       要素賦存，所得
parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    e_r         資源の賦存量
    inc0        ベンチマーク所得
;
*       ベンチマークの賦存量 (供給量)
e_k = sam("pk","cons") / pk0;
e_l = sam("pl","cons") / pl0;
e_r = sam("pr","cons") / pr0;

*       ベンチマークの所得
inc0 = - sam("pu","cons");

display e_l, e_k, e_r, inc0;

*       ----------------------------------------------------------------------
*       税金についてのパラメータ．
$ontext

・税は 3 種類．1) x 財に対する生産税，2) 労働課税，3 ) 一括税
・1 と 2 については従価税として扱う．
・従価税率はベンチマークデータより導出できる (税額 / タックスベース)．
・労働課税率は二つの部門で異なる値．

$offtext
parameter
    tl0(j)      ベンチマークにおける j 部門の労働に対する税率
    tk0(j)      ベンチマークにおける j 部門の資本に対する税率
    tq0(j)      ベンチマークにおける j 部門の生産物に対する税率

    tl(j)       ベンチマークにおける j 部門の労働に対する税率
    tk(j)       ベンチマークにおける j 部門の資本に対する税率
    tq(j)       ベンチマークにおける j 部門の生産物に対する税率

    vtlump0     ベンチマークにおける一括税
;
*       税率 = 税額 / タックスベース
tl0(j) = sam("vtl",j) / sam("pl",j);
tq0("x") = - sam("vtq","x") / sam("px","x");
tq0("y") = - sam("vtq","y") / sam("py","y");
tq0("z") = - sam("vtq","z") / sam("pz","z");
tk0(j) = 0;

display tl0, tq0;

tl(j) = tl0(j);
tk(j) = tk0(j);
tq(j) = tq0(j);

*       一括税
vtlump0 = sam("vtlump","gov");
display vtlump0;

*       ----------------------------------------------------------------------
*       CO2
display "@ CO2";

parameter
    z0          ベンチマークにおける z 財の量
    delta       排出係数
    co20        ベンチマークにおけるCO2排出量
    co2lim      排出量へのキャップ
    pzc0        ベンチマークにおける z 財の価格（排出権価格込み）
    tco2        外生的に与える炭素税率
;
$ontext
単純化のため排出係数は1と置く。
$offtext
z0 = q0("z");
delta = 1;
co20 = delta * z0;
tco2 = 0;

$ontext
ベンチマーク均衡ではCO2の規制はないとする。よって、CO2へのキャップは binding に
ならないように大きい値（ここではベンチマークの排出量の10倍）にしておく。
$offtext
co2lim = co20 * 10;

$ontext
ベンチマーク均衡ではCO2へのキャップは biding にはならないので、排出権価格は0にな
る。よって、z 財の価格に等しい。
$offtext
pzc0 = p0("z");

display z0, delta, co20, tco2, pzc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

*       代替の弾力性：
parameter
    sig_1(j)    EOS: 部門 j におけるEOS
    sig_2(j)    EOS: 部門 j におけるEOS
    sig_u       EOS: 効用関数内の EOS                   / 2.0 /
    sig_g       EOS: 政府支出における EOS               / 0.0 /
;
sig_1("x") = 1;
sig_2("x") = 2;
sig_1("y") = 1;
sig_2("y") = 2;
sig_1("z") = 0.5;
sig_2("z") = 1;

display sig_1, sig_2, sig_u, sig_g;

$ontext
以下は要素賦存量と政府支出レベルを変更するシミュレーションをおこなうためのパラメー
タ。
$offtext
parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
    s_r         資源賦存量のスケールファクター  / 1.0 /
    s_lump      一括税のスケールファクター      / 1.0 /
    s_gov       政府支出のスケールファクター    / 1.0 /
;
display s_l, s_k, s_r, s_lump, s_gov;

$ontext
排出権収入の処理

CO2にキャップを課すと排出権の売却収入が政府にはいってくる。その排出権収入の処理
の仕方

[fl_lump]
・これが非ゼロの場合には，排出権収入をそのまま一括で家計に渡す。
・厳密に言うと、排出権収入が入ってきても政府支出が一定になるように、家計への一括
　税を減税する。

[fl_qx]

・これが非ゼロの場合には，排出権収入の分だけ、x 部門の生産にかかっている税を減税
　する。
・厳密に言うと、排出権収入が入ってきても政府支出が一定になるように、x 部門の生産
　への税を減税する。

$offtext
parameter
    fl_lump     非ゼロなら排出権収入を一括で家計に還元
    fl_qx       非ゼロなら排出権収入の分だけ生産への課税を減税
;
fl_lump = yes;
fl_qx = no;

$ontext
MPSGEモデル
$offtext

$ontext

*       モデル名
$model:ex_carbon

*       アクティビティー変数 (sector) の宣言
$sectors:
    q(j)        ! j 部門のアクティビティーレベル
    u           ! u 部門の生産量 (効用水準)
    g           ! g 部門の生産量 (政府支出水準)
    azc         ! z財と排出権の統合

*       価格変数 (commodity) の宣言
$commodities:
    p(j)        ! j 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格
    pr          ! 資源の価格
    pg          ! 政府支出財の価格
    pzc         ! 排出権の価格込みの z 財の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得
    gov         ! 政府の所得

*       補助変数の宣言
$auxiliary:
    t_co2               ! Carbon tax rate
    t_lump$fl_lump      ! Adjustment by lump-sum tax
    t_qx$fl_qx          ! Adjustment by output tax

$prod:azc  s:0
    o:pzc  q:z0
    i:p("z")    q:z0     p:p0("z")  a:gov  n:t_co2

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:q("x")  s:0  lkz:sig_1("x")  kz(lkz):sig_2("x")
    o:p("x")    q:q0("x")   p:((1-tq0("x"))*p0("x"))  a:gov  t:tq("x")$(not fl_qx)
+   n:t_qx$fl_qx  m:tq0("x")$fl_qx
    i:p("y")    q:int0("y","x")    p:p0("y")
    i:pl        q:ld0("x")  p:((1+tl0("x"))*pl0)  lkz:  a:gov  t:tl("x")
    i:pk        q:kd0("x")  p:pk0  kz:
    i:pzc       q:int0("z","x")    p:pzc0  kz:

*       部門 y の生産関数
$prod:q("y")  s:0  lkz:sig_1("y")  kz(lkz):sig_2("y")
    o:p("y")    q:q0("y")   p:((1-tq0("y"))*p0("y"))  a:gov  t:tq("y")
    i:p("x")    q:int0("x","y")    p:p0("x")
    i:pl        q:ld0("y")  p:((1+tl0("y"))*pl0)  lkz:  a:gov  t:tl("y")
    i:pk        q:kd0("y")  p:pk0  kz:
    i:pzc       q:int0("z","y")    p:pzc0  kz:

*       部門 z の生産関数
$prod:q("z")  s:sig_1("z")  kl:sig_2("z")
    o:p("z")    q:q0("z")   p:((1-tq0("z"))*p0("z"))  a:gov  t:tq("z")
    i:pr        q:rd0("z")  p:pr0
    i:pl        q:ld0("z")  p:((1+tl0("z"))*pl0)  kl:  a:gov  t:tl("z")
    i:pk        q:kd0("z")  p:pk0  kl:

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:p("x")    q:cd0("x")
    i:p("y")    q:cd0("y")
    i:pzc       q:cd0("z")  p:pzc0

*       部門 g の生産関数 (政府支出)
$prod:g  s:sig_g
    o:pg        q:g0
    i:p(j)      q:gd0(j)  p:p0(j)

*       ----------------------------------------------------------------------
*       予算制約

*       民間消費者の予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)
    e:pr        q:(e_r*s_r)
    e:pg        q:(-vtlump0*s_lump)  r:t_lump$fl_lump

*       政府の予算制約
$demand:gov
    d:pg
    e:pg        q:(vtlump0*s_lump)  r:t_lump$fl_lump

*       ----------------------------------------------------------------------
*       Constraints:

$constraint:t_lump$fl_lump
    g =e= s_gov;

$constraint:t_qx$fl_qx
    g =e= s_gov;

$constraint:t_co2
    t_co2*p("z") / (pu*delta) =e= tco2;

*       ----------------------------------------------------------------------
*       Report 用の変数
$report:
    v:z_ld(j)   i:pl    prod:q(j)       ! j 部門の労働需要
    v:z_kd(j)   i:pk    prod:q(j)       ! j 部門の資本需要
    v:z_rd(j)   i:pr    prod:q(j)       ! j 部門の資源需要

$offtext

*       preprocessor の呼び出し
$ontext
$sysinclude mpsgeset model_name

という書き方をする．
$offtext
$sysinclude mpsgeset ex_carbon

*       変数の初期値
t_co2.l = 0;

*       変数の初期値
t_lump.l = 1;
t_qx.l = 1;

*       Bounds:
$ontext
MPSGEでは変数のデフォールトの下限は 0．補助変数 t_lump は負の値もとるので下限を
変更しておく．t_qx も負の値をとってよいことにしておく。
$offtext
t_lump.lo = -inf;
t_qx.lo = -inf;

*       結果表示用の index の定義．
set     pol     / u, g, qx, qy, qz, px, py, pz, pk, pl, pr, co2 /
        case            Cases           / b, c1, c2, c3, c4 /
        c_case(case)    Current case
;
c_case(case) = no;

*       結果表示用のパラメータ
parameter
    res_l               変数
    res_pc              変数の変化率（%）
    res_summary         まとめ
;
$macro export_results(case) \
    res_l("u",case) = u.l; \
    res_l("g",case) = g.l; \
    res_l("qx",case) = q.l("x"); \
    res_l("qy",case) = q.l("y"); \
    res_l("qz",case) = q.l("z"); \
    res_l("px",case) = p.l("x") / pu.l; \
    res_l("py",case) = p.l("y") / pu.l; \
    res_l("pz",case) = p.l("z") / pu.l; \
    res_l("pk",case) = pk.l / pu.l; \
    res_l("pl",case) = pl.l / pu.l; \
    res_l("pr",case) = pr.l / pu.l; \
    res_l("co2",case) = delta*z0*azc.l; \
    res_l("pco2",case) = tco2; \
    res_l("pco2_",case) = t_co2.l*p.l("z") / (pu.l*delta); \
    res_l("t_lump",case) = t_lump.l; \
    res_l("t_qx",case) = t_qx.l; \
    res_l("rev_perm",case) = azc.l*z0*delta*tco2; \
    res_l("rev_perm_",case) = t_co2.l*p.l("z")*azc.l*z0 / pu.l; \
    res_l("tax_lump",case) = \
        pg.l*vtlump0*s_lump*(t_lump.l$fl_lump + 1$(not fl_lump)) / pu.l; \
    res_l("tax_qx",case) = \
        tq0("x")*(t_qx.l$fl_qx + 1$(not fl_qx))*p.l("x")*q.l("x")*q0("x") / pu.l;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data の下で均衡が成立しているかをチェックする．

これには iteration の回数をゼロとして解く．

model_name.iterlim = 0;

とすると iteration 回数を 0 に設定できる．
$offtext

ex_carbon.iterlim = 0;
$include ex_carbon.gen
solve ex_carbon using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

$ontext
iteration 回数を増加して解く．

モデル，データが適切ならば，変数の初期値がそのまま解となるはず．
$offtext

ex_carbon.iterlim = 10000;
$include ex_carbon.gen
solve ex_carbon using mcp;

$ontext
ベンチマークの変数値．
"b" はベンチマークの値を表す．
$offtext

*       結果をパラメータに代入．
export_results("b");

*       ----------------------------------------------------------------------
*       Check: モデルのチェック
display "@ Check: モデルのチェック";

*       10% ずつの増加．
s_k = 1.5;
s_l = 1.5;
s_r = 1.5;
s_lump = 1.5;
s_gov = 1.5;

$include ex_carbon.gen
solve ex_carbon using mcp;

s_k = 1;
s_l = 1;
s_r = 1;
s_lump = 1;
s_gov = 1;

$include ex_carbon.gen
solve ex_carbon using mcp;

*       ======================================================================
*       以下はシミュレーションの例．
display "@ 以下はシミュレーションの例．";

*       ----------------------------------------------------------------------
*       C1: 排出規制（5%削減）+ 一括の還元
display "@ C1: 排出規制（5%削減）+ 一括の還元";
$ontext

+ tco2 に炭素税率を設定する。
  + ここでは CO2 が5%減少するような炭素税率を設定する。
  + ex_permit.gms において C1 で求めた排出権価格の値を税率に設定。
+ 炭素税収は家計に一括で還元する。
+ ex_permit.gms の C1 シナリオと全く同じ均衡になる。
  + 排出権取引と炭素税の同値性の証明

$offtext

tco2 = 0.20729982;
fl_lump = 1;
fl_qx = 0;
t_lump.l = 1;
t_qx.l = 1;

$include ex_carbon.gen
solve ex_carbon using mcp;

*       結果をパラメータに代入．
export_results("c1");

*       ----------------------------------------------------------------------
*       C2: 排出規制（10%削減）+ 一括の還元
display "@ C2: 排出規制（10%削減）+ 一括の還元";
$ontext

+ tco2 に炭素税率を設定する。
  + ここでは CO2 が10%減少するような炭素税率を設定する。
+ あとは C1 と同じ。

$offtext

tco2 = 0.37251912;
fl_lump = 1;
fl_qx = 0;
t_lump.l = 1;
t_qx.l = 1;

$include ex_carbon.gen
solve ex_carbon using mcp;

*       結果をパラメータに代入．
export_results("c2");

*       ----------------------------------------------------------------------
*       C3: 排出規制（5%削減）+ 生産税の減税
display "@ C3: 排出規制（5%削減） + 生産税の減税";
$ontext

+ tco2 に炭素税率を設定する。
  + ここでは CO2 が5%減少するような炭素税率を設定する。
  + ex_permit.gms において C1 で求めた排出権価格の値を税率に設定。
+ 炭素税収分だけ x 部門への税を減税。
+ ex_permit.gms の C3 シナリオと全く同じ均衡になる。
  + 排出権取引と炭素税の同値性の証明

$offtext

tco2 = 0.19374563;
fl_lump = 0;
fl_qx = 1;
t_lump.l = 1;
t_qx.l = 1;

$include ex_carbon.gen
solve ex_carbon using mcp;

*       結果をパラメータに代入．
export_results("c3");

*       ----------------------------------------------------------------------
*       C4: 排出規制（10%削減）+ 生産税の減税
display "@ C4: 排出規制（10%削減）+ 生産税の減税";
$ontext

+ tco2 に炭素税率を設定する。
  + ここでは CO2 が10%減少するような炭素税率を設定する。
+ あとは C3 と同じ。

$offtext

tco2 = 0.36179167;
fl_lump = 0;
fl_qx = 1;
t_lump.l = 1;
t_qx.l = 1;

$include ex_carbon.gen
solve ex_carbon using mcp;

*       結果をパラメータに代入．
export_results("c4");

*       ----------------------------------------------------------------------
*       変数の変化率の導出．
$ontext
ベンチマークからの変数の変化率を導出．

$offtext
res_pc(pol,case)$res_l(pol,"b")
    = 100 * (res_l(pol,case) / res_l(pol,"b") - 1);

res_summary("co2",case) = res_pc("co2",case);
res_summary("pco2",case) = res_l("pco2",case);
res_summary("t_lump",case) = res_l("t_lump",case);
res_summary("t_qx",case) = res_l("t_qx",case);
res_summary("u",case) = res_pc("u",case);

*       結果の表示．
display res_l;
display res_pc, res_summary;

execute_unload "ex_permit.gdx", res_l, res_pc, res_summary;

* --------------------
* Local Variables:
* mode: gams
* fill-column: 70
* coding: utf-8
* End:
