$title ex_dem_01.gms: $demandブロックで消費 (効用関数) を表現するケース
display "@ ex_dem_01.gms: $demandブロックで消費 (効用関数) を表現するケース";
$ontext
Time-stamp:     <2023-05-01 15:53:01 st>
First-written:  <2010/03/12>

$demanブロックで消費を表すケース．
・$demand ブロックで消費を表すケースでは，u 部門はなくなる．
・二つのケース
(ケース1) $demand ブロックで消費を表すケース → モデル名 ex_dem_01
(ケース2) u 部門で消費を表すケース → モデル名 ex_prod_01


□ SAM (ベンチマークデータ)
           |         Sectors         |  Consumers
Commoditiy |          (部門)         |   (消費者)
   (財)    |    x       y        u   |    cons
-----------------------------------------------------
       px  |  100             -100   |
       py  |          100     -100   |
       pu  |                   200   |    -200
       pl  |  -25     -75            |     100
       pk  |  -75     -25            |     100
-----------------------------------------------------


□ 部門 x の生産関数

         x
        / \ <- sig_x
       /   \
      /     \
     k      l

・CES 型関数．
・投入物は資本，労働のみ．
・代替の弾力性は sig_x．

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
               x       y        u     cons
px           100       0     -100        0
py             0     100     -100        0
pu             0       0      200     -200
pl           -25     -75        0      100
pk           -75     -25        0      100
;
display sam;

parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
display px0, py0, pu0, pk0, pl0;

parameter
    x0          x 財のベンチマーク生産量
    y0          y 財のベンチマーク生産量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
;

x0 = sam("px","x") / px0;
y0 = sam("py","y") / py0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;

display x0, y0, kx0, ky0, lx0, ly0;

parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
;
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;

display u0, cx0, cy0;

parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    inc0        ベンチマーク所得
;
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;
inc0 = - sam("pu","cons");

display e_l, e_k, inc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

parameter
    sig_x       EOS: 部門 x における資本・労働の EOS    / 0.5 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
;
display sig_x, sig_y;

parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
;
display s_l, s_k;

*       ----------------------------------------------------------------------
*       u 部門で消費を表現するモデル
$ontext
$model:ex_prod_01

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量
    u           ! u 部門の生産量 (効用水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格
    
*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得
    
*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:x  s:sig_x
    o:px        q:x0
    i:pk        q:kx0  p:pk0
    i:pl        q:lx0  p:pl0

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0

*       ----------------------------------------------------------------------
*       予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)

$offtext

$sysinclude mpsgeset ex_prod_01


*       ----------------------------------------------------------------------
*       demand ブロックで消費を表現するモデル
$ontext
$model:ex_dem_01

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pk          ! 労働の価格
    pl          ! 資本の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:x  s:sig_x
    o:px        q:x0
    i:pk        q:kx0  p:pk0
    i:pl        q:lx0  p:pl0

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       ----------------------------------------------------------------------
*       予算制約
$demand:cons  s:sig_u
    d:px        q:cx0   p:px0
    d:py        q:cy0   p:py0
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)

$report:
    v:z_u       w:cons

$offtext

$sysinclude mpsgeset ex_dem_01

*       結果表示用のパラメータ
parameter
    chk_results
;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data が 均衡となっているかをチェックする．
$offtext

ex_prod_01.iterlim = 0;
$include ex_prod_01.gen
solve ex_prod_01 using mcp;

ex_dem_01.iterlim = 0;
$include ex_dem_01.gen
solve ex_dem_01 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

ex_prod_01.iterlim = 10000;
$include ex_prod_01.gen
solve ex_prod_01 using mcp;

*       ケース1
chk_results("x","b0") = x.l;
chk_results("y","b0") = y.l;
chk_results("u","b0") = u.l;
chk_results("px","b0") = px.l / px.l;
chk_results("py","b0") = py.l / px.l;
chk_results("pk","b0") = pk.l / px.l;
chk_results("pl","b0") = pl.l / px.l;

ex_dem_01.iterlim = 10000;
$include ex_dem_01.gen
solve ex_dem_01 using mcp;

*       ケース2
chk_results("x","b1") = x.l;
chk_results("y","b1") = y.l;
chk_results("u","b1") = z_u.l;
chk_results("px","b1") = px.l / px.l;
chk_results("py","b1") = py.l / px.l;
chk_results("pk","b1") = pk.l / px.l;
chk_results("pl","b1") = pl.l / px.l;

*       ----------------------------------------------------------------------
*       労働賦存量の 10% の増加:
display "@ 労働賦存量の 10% の増加:";

s_l = 1.1;

ex_prod_01.iterlim = 10000;
$include ex_prod_01.gen
solve ex_prod_01 using mcp;

*       ケース1
chk_results("x","cf0") = x.l;
chk_results("y","cf0") = y.l;
chk_results("u","cf0") = u.l;
chk_results("px","cf0") = px.l / px.l;
chk_results("py","cf0") = py.l / px.l;
chk_results("pk","cf0") = pk.l / px.l;
chk_results("pl","cf0") = pl.l / px.l;

ex_dem_01.iterlim = 10000;
$include ex_dem_01.gen
solve ex_dem_01 using mcp;

*       ケース2
chk_results("x","cf1") = x.l;
chk_results("y","cf1") = y.l;
chk_results("u","cf1") = z_u.l;
chk_results("px","cf1") = px.l / px.l;
chk_results("py","cf1") = py.l / px.l;
chk_results("pk","cf1") = pk.l / px.l;
chk_results("pl","cf1") = pl.l / px.l;

*       二つのケースで同じ計算結果になるかチェック．
display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 80
* coding: utf-8
* End:
