$title ex_full_01.gms: サンプルのプログラム．
display "@ ex_full_01.gms: サンプルのプログラム．";
$ontext
Time-stamp:     <2022-01-29 11:20:23 st>
First-written:  <2010/03/12>

□ 説明
・このプログラムには他のサンプルの要素をまとめていれてある．
・他のプログラムより説明を詳しくしている．


□ SAM (ベンチマークデータ)
           |             Sectors                    |     Consumers
Commoditiy |              (部門)                    |      (消費者)
   (財)    |    x       y       z       u      g    |    cons     gov
-----------------------------------------------------------------------
       px  |  140     -10            -110     -20   |
       py  |  -15     115             -80     -20   |
       pz  |    5              15     -20           |
       pu  |                          210           |    -210
       pl  |  -30     -75      -5                   |     110
       pk  |  -75     -25     -10                   |     110
       pg  |                                   40   |             -40
     ----------------------------------------------------------------
      vtls |                                        |     -10      10
      vtl  |  -15      -5                           |              20
      vtx  |  -10                                   |              10
-----------------------------------------------------------------------

□ 政府

・政府は労働とx財に税金をかけている．また，消費者から一括税を徴収
　している．
・政府は税収を政府支出に支出する．
・政府支出は「政府支出財」の購入という形式で表現される．
・政府支出財の価格は pg．
・政府支出財は政府支出財部門 (g部門) によって生産される．

□ 生産関数

[x 部門]
  x     z
  \     /
   \   /
    \ / <- eta_x
     |
    / \ <- 0
   /   \
  /     \
 y     / \ <- sig_x
      /   \
     /     \
     k      l

[y 部門]
     y
    / \ <- 0
   /   \
  /     \
 x     / \ <- sig_y
      /   \
     /     \
     k      l

[z 部門]
     z
    / \ <- sig_z
   /   \
  /     \
 k       l

[u 部門]
     u
    / \ <- sig_u
   /   \
  /     \
 x       y

□ 政府支出財部門の生産関数

     g
    / \ <- sig_g
   /   \
  /     \
 x      y

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
               x       y       z        u       g     cons     gov
px           140     -10       0     -110     -20        0       0
py           -15     115       0      -80     -20        0       0
pz             5       0      15      -20       0        0       0
pu             0       0       0      210       0     -210       0
pl           -30     -75      -5        0       0      110       0
pk           -75     -25     -10        0       0      110       0
pg             0       0       0        0      40        0     -40
vtls           0       0       0        0       0      -10      10
vtl          -15      -5       0        0       0        0      20
vtx          -10       0       0        0       0        0      10
;
display sam;

$ontext
ベンチマークにおける価格．

・ベンチマークにおける財・要素の市場価格は全て1と仮定する．
・仮に 1 以外の値を仮定したとしても，結果 (変数の変化率) は変わらな
　い．
・市場価格は 1 だが，agent 価格は 1 とは限らない．
$offtext
parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pz0         z 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pz0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
display px0, py0, pz0, pu0, pk0, pl0;

*       ----------------------------------------------------------------------
*       生産サイド

$ontext
ベンチマークの数量．

・SAM の金額データとベンチマーク価格より，ベンチマークの数量が導出でき
　る．
$offtext
parameter
    x0          x 部門の x 財のベンチマーク生産量
    y0          y 部門の y 財のベンチマーク生産量
    z0          z 部門の z 財のベンチマーク生産量
    zx0         x 部門の z 財のベンチマーク生産量
    yx0         x 部門の y 財のベンチマーク投入量
    xy0         y 部門の x 財のベンチマーク投入量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    kz0         z 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
    lz0         z 部門における労働のベンチマーク投入量
;

*       数量 = 金額 / 価格
x0 = sam("px","x") / px0;
y0 = sam("py","y") / py0;
z0 = sam("pz","z") / pz0;
zx0 = sam("pz","x") / pz0;
yx0 = - sam("py","x") / py0;
xy0 = - sam("px","y") / px0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
kz0 = - sam("pk","z") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;
lz0 = - sam("pl","z") / pl0;

display x0, y0, z0, zx0, yx0, xy0, kx0, ky0, kz0, lx0, ly0, lz0;

*       ----------------------------------------------------------------------
*       消費サイド．
parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
    cz0         z 財のベンチマーク消費量
;
*       ベンチマークの効用，消費量
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;
cz0 = - sam("pz","u") / pz0;

display u0, cx0, cy0, cz0;

*       ----------------------------------------------------------------------
*       政府
parameter
    pg0         ベンチマークにおける政府支出財の価格
    g0          ベンチマークにおける政府支出
    xg0         ベンチマークにおける x 財に対する政府支出需要
    yg0         ベンチマークにおける y 財に対する政府支出需要
;
*       これについてもベンチマーク価格は 1 と仮定．
pg0 = 1;

*       ベンチマークの生産量，投入量
g0 = sam("pg","g") / pg0;
xg0 = - sam("px","g") / px0;
yg0 = - sam("py","g") / py0;

display pg0, g0, xg0, yg0;

*       ----------------------------------------------------------------------
*       要素賦存，所得
parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    inc0        ベンチマーク所得
;
*       ベンチマークの賦存量 (供給量)
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;

*       ベンチマークの所得
inc0 = - sam("pu","cons");

display e_l, e_k, inc0;

*       ----------------------------------------------------------------------
*       税金についてのパラメータ．
$ontext
・税は 3 種類．1) x 財に対する生産税，2) 労働課税，3 ) 一括税
・1 と 2 については従価税として扱う．
・従価税率はベンチマークデータより導出できる (税額 / タックスベース)．
・労働課税率は二つの部門で異なる値．

$offtext
parameter
    tlx0        ベンチマークにおける x 部門の労働に対する税率
    tly0        ベンチマークにおける y 部門の労働に対する税率
    tlz0        ベンチマークにおける z 部門の労働に対する税率
    tx0         ベンチマークにおける x 部門の生産物に対する税率

    tlx         x 部門の労働に対する税率
    tly         y 部門の労働に対する税率
    tlz         z 部門の労働に対する税率
    tx          x 部門の生産物に対する税率
    vtls0       ベンチマークにおける一括税
;
*       税率 = 税額 / タックスベース
tlx0 = sam("vtl","x") / sam("pl","x");
tly0 = sam("vtl","y") / sam("pl","y");
tlz0 = sam("vtl","z") / sam("pl","z");
tx0 = - sam("vtx","x") / sam("px","x");
display tlx0, tly0, tlz0, tx0;

tlx = tlx0;
tly = tly0;
tlz = tlz0;
tx = tx0;

*       一括税
vtls0 = sam("vtls","gov");
display vtls0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

*       代替の弾力性：
parameter
    sig_x       EOS: 部門 x における資本・労働の EOS    / 0.5 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_z       EOS: 部門 z における資本・労働の EOS    / 0.2 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
    sig_g       EOS: 政府支出における EOS               / 0.0 /
;
display sig_x, sig_y, sig_z, sig_u;

$ontext
以下は要素賦存量と政府支出レベルを変更するシミュレーションをおこなうた
めのパラメータ。
$offtext
parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
    s_lump      一括税のスケールファクター      / 1.0 /
    s_gov       政府支出のスケールファクター    / 1.0 /
;
display s_l, s_k, s_lump, s_gov;

$ontext
政策の場合分け．

[fl_lump]
これが非ゼロの場合には，政府支出が一定となるように，税収を一括税で調整
する．

[fl_uni]
これが非ゼロの場合には，政府支出が一定となるように，税収を uniform な
労働課税で調整する．

$offtext
parameter
    fl_lump     非ゼロなら政府支出が一定となるように一括税で調整
    fl_uni      非ゼロなら Uniform な労働課税
;
fl_lump = yes;
fl_uni = no;

$ontext
MPSGEモデル

$offtext

$ontext

*       モデル名
$model:ex_full_01

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門のアクティビティーレベル
    y           ! y 部門の生産量
    z           ! z 部門の生産量
    u           ! u 部門の生産量 (効用水準)
    g           ! g 部門の生産量 (政府支出水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pz          ! z 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格
    pg          ! 政府支出財の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得
    gov         ! 政府の所得

*       補助変数の宣言
$auxiliary:
    tlump$fl_lump       ! Lump-sum tax
    tluni$fl_uni        ! Uniform な労働税率

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:x  s:0  kl:sig_x
    o:px        q:x0   p:((1-tx0)*px0)  a:gov  t:tx
    o:pz        q:zx0  p:pz0
    i:py        q:yx0  p:py0
    i:pk        q:kx0  p:pk0  kl:
    i:pl        q:lx0  p:((1+tlx0)*pl0)  kl:
+   a:gov  t:tlx$(not fl_uni)  n:tluni$fl_uni

*       部門 y の生産関数
$prod:y  s:0  kl:sig_y
    o:py        q:y0
    i:px        q:xy0  p:px0
    i:pk        q:ky0  p:pk0  kl:
    i:pl        q:ly0  p:((1+tly0)*pl0) kl:
+   a:gov       t:tly$(not fl_uni)  n:tluni$fl_uni

*       部門 z の生産関数
$prod:z  s:sig_z
    o:pz        q:z0
    i:pk        q:kz0  p:pk0
    i:pl        q:lz0  p:((1+tlz0)*pl0)
+   a:gov       t:tlz$(not fl_uni)  n:tluni$fl_uni

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0
    i:pz        q:cz0  p:pz0

*       部門 g の生産関数 (政府支出)
$prod:g  s:sig_g
    o:pg        q:g0
    i:px        q:xg0  p:px0
    i:py        q:yg0  p:py0

*       ----------------------------------------------------------------------
*       予算制約

*       民間消費者の予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)
    e:pg        q:(-vtls0*s_lump)  r:tlump$fl_lump

*       政府の予算制約
$demand:gov
    d:pg
    e:pg        q:(vtls0*s_lump)  r:tlump$fl_lump

*       ----------------------------------------------------------------------
*       Constraints:

$constraint:tlump$fl_lump
    g =e= s_gov;

$constraint:tluni$fl_uni
    g =e= s_gov;

*       ----------------------------------------------------------------------
*       Report 用の変数
$report:
    v:z_lx      i:pl    prod:x          ! x 部門の労働需要
    v:z_ly      i:pl    prod:y          ! y 部門の労働需要
    v:z_lz      i:pl    prod:z          ! z 部門の労働需要
    v:z_kx      i:pk    prod:x          ! x 部門の資本需要
    v:z_ky      i:pk    prod:y          ! y 部門の資本需要
    v:z_kz      i:pk    prod:z          ! z 部門の資本需要

$offtext

*       preprocessor の呼び出し
$ontext
$sysinclude mpsgeset model_name

という書き方をする．
$offtext
$sysinclude mpsgeset ex_full_01

*       変数の初期値
tlump.l = 1;
tluni.l = 0;

*       Bounds:
$ontext
MPSGEでは変数のデフォールトの下限は 0．補助変数 tlump と tluni は負の値
もとるので下限を変更しておく．
$offtext
tlump.lo = -inf;
tluni.lo = -inf;

*       結果表示用の index の定義．
set     pol     / u, g, x, y, z, px, py, pz, pk, pl, pr /
        case            Cases           / b, c1*c4 /
        c_case(case)    Current case
;
c_case(case) = no;

*       結果表示用のパラメータ
parameter
    res_l
    chk_results
;
*       結果表示用のマクロの定義
$macro export_results(case) \
    res_l("u",case) = u.l; \
    res_l("g",case) = g.l; \
    res_l("x",case) = x.l; \
    res_l("y",case) = y.l; \
    res_l("z",case) = z.l; \
    res_l("px",case) = px.l / pu.l; \
    res_l("py",case) = py.l / pu.l; \
    res_l("pz",case) = pz.l / pu.l; \
    res_l("pk",case) = pk.l / pu.l; \
    res_l("pl",case) = pl.l / pu.l;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data の下で均衡が成立しているかをチェックする．

これには iteration の回数をゼロとして解く．

model_name.iterlim = 0;

とすると iteration 回数を 0 に設定できる．
$offtext

ex_full_01.iterlim = 0;
$include ex_full_01.gen
solve ex_full_01 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

$ontext
iteration 回数を増加して解く．

モデル，データが適切ならば，変数の初期値がそのまま解となるはず．
$offtext

ex_full_01.iterlim = 10000;
$include ex_full_01.gen
solve ex_full_01 using mcp;

$ontext
ベンチマークの変数値．
"b" はベンチマークの値を表す．
$offtext

*       マクロの実行
export_results("b");

*       ======================================================================
*       以下はシミュレーションの例．
display "@ 以下はシミュレーションの例．";

*       ----------------------------------------------------------------------
*       C1: 賦存量の比例的増加．
display "@ C1: 賦存量の比例的増加．";
$ontext

賦存量にあたる外生変数 (パラメータ) を比例的 (10%) に増加させるショッ
ク．

[Check]
・生産量，効用等の実質値はどう変化するか？
・価格 (名目変数) はどう変化するか？

$offtext

*       10% ずつの増加．
s_k = 1.1;
s_l = 1.1;
s_lump = 1.1;
s_gov = 1.1;

$include ex_full_01.gen
solve ex_full_01 using mcp;

s_k = 1;
s_l = 1;
s_lump = 1;
s_gov = 1;

*       結果をパラメータに代入．
export_results("c1");

*       ----------------------------------------------------------------------
*       C2: 労働課税率を2倍に上昇させる．
display "@ C2: 労働課税率を2倍に上昇させる．";
$ontext
既存の労働課税率を 2 倍に上昇させるショック．

[Check]
・消費者の効用はどう変化するか？
・元々，労働課税率は x 部門が高い．各部門の生産量にどのような影響があ
　るか？
$offtext

tlx = tlx0 * 2;
tly = tly0 * 2;
tlz = tlz0 * 2;

$include ex_full_01.gen
solve ex_full_01 using mcp;

tlx = tlx0;
tly = tly0;
tlz = tlz0;

export_results("c2");

*       ----------------------------------------------------------------------
*       C3: Uniform な労働税率に変更．
display "@ C3: Uniform な労働税率に変更．";
$ontext
ベンチマークでは各部門における労働課税率は異なっている．これを全ての部
門に対し一様な税率に変更するショック．

一様な税率の水準は政府支出が一定となるように決定する．

[Check]
・効用はどう変化するか？改善するか？
$offtext

fl_lump = no;
fl_uni = yes;

$include ex_full_01.gen
solve ex_full_01 using mcp;

fl_lump = yes;
fl_uni = no;

export_results("c3");

*       ----------------------------------------------------------------------
*       C4: 一括税以外の税を撤廃．
display "@ C4: 一括税以外の税を撤廃．";
$ontext
一括税以外の税を撤廃する (税率をゼロにする) ショック．

一括税の額は政府支出が一定となるように調整する．

[Check]
・効用はどう変化するか？改善するか？
$offtext

tx = 0;
tlx = 0;
tly = 0;
tlz = 0;

$include ex_full_01.gen
solve ex_full_01 using mcp;

tx = tx0;
tlx = tlx0;
tly = tly0;
tlz = tlz0;

export_results("c4");

*       ----------------------------------------------------------------------
*       変数の変化率の導出．
$ontext
ベンチマークからの変数の変化率を導出．

$offtext
chk_results(pol,case)$res_l(pol,"b")
    = 100 * (res_l(pol,case) / res_l(pol,"b") - 1);

*       結果の表示．
display res_l;
display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 70
* coding: utf-8
* End:
