$title ex_mpsge_mcp.gms 同じモデルを MCP と MPSGE で記述するプログラム．
display "@ ex_mpsge_mcp.gms 同じモデルを MCP と MPSGE で記述するプログラム．";
$ontext
Time-stamp:     <2023-05-01 16:04:50 st>
First-written:  <2010/03/12>

□ 説明

・同じモデルを MCP と MPSGE で記述するプログラム．
・MPSGEの記述を数式に変換したもの．

□ SAM (ベンチマークデータ)
           |             Sectors                    |     Consumers
Commoditiy |              (部門)                    |      (消費者)
   (財)    |    x       y       z       u      g    |    cons     gov
-----------------------------------------------------------------------
       px  |  140     -10       0    -110     -20   |       0       0
       py  |  -15     115       0     -80     -20   |       0       0
       pz  |    5       0      15     -20       0   |       0       0
       pu  |    0       0       0     210       0   |    -210       0
       pl  |  -30     -75      -5       0       0   |     110       0
       pk  |  -75     -25     -10       0       0   |     110       0
       pg  |    0       0       0       0      40   |       0     -40
     ----------------------------------------------------------------
     vtls  |    0       0       0       0       0   |     -10      10
      vtl  |  -15      -5       0       0       0   |       0      20
      vtx  |  -10       0       0       0       0   |       0      10
-----------------------------------------------------------------------

□ 政府

・政府は労働とx財に税金をかけている．また，消費者から一括税を徴収
　している．
・政府は税収を政府支出に支出する．
・政府支出は「政府支出財」の購入という形式で表現される．
・政府支出財の価格は pg．
・政府支出財は政府支出財部門 (g部門) によって生産される．

□ 生産関数

[x 部門]
  x     z
  \     /
   \   /
    \ / <- eta_q("x")
     |
    / \ <- 0
   /   \
  /     \
 y     / \ <- sig_kl("x")
      /   \
     /     \
     k     l


[y 部門]
     y
    / \ <- 0
   /   \
  /     \
 x     / \ <- sig_kl("y")
      /   \
     /     \
     k     l


[z 部門]
     z
    / \ <- sig_kl("z")
   /   \
  /     \
 k      l


[u 部門]
     u
    / \ <- sig_u
   /   \
  /     \
 x   y   z 


□ 政府支出財部門の生産関数

     g
    / \ <- sig_g
   /   \
  /     \
 x      y


$offtext

*       ======================================================================
$stitle ベンチマーク・データ:
*       ======================================================================
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
               x       y       z        u       g     cons     gov
x            140     -10       0     -110     -20        0       0
y            -15     115       0      -80     -20        0       0
z              5       0      15      -20       0        0       0
u              0       0       0      210       0     -210       0
l            -30     -75      -5        0       0      110       0
k            -75     -25     -10        0       0      110       0
pg             0       0       0        0      40        0     -40
vtls           0       0       0        0       0      -10      10
vtl          -15      -5       0        0       0        0      20
vtx          -10       0       0        0       0        0      10
;
display      sam;

$ontext
インデックスの定義

+ 部門を表現するインデックス
+ 生産要素を表現するインデックス

を定義．
$offtext

set     i       財      / x, y, z /
        f       要素    / k, l /
        lab(f)  労働    / l /
;
alias (i,ii), (i,j), (f,ff);

$ontext
ベンチマークにおける価格．

・ベンチマークにおける財・要素の市場価格は全て1と仮定する．
・仮に 1 以外の値を仮定したとしても，結果 (シミュレーション結果におけ
　る変数の変化率) が変わらない．そこで単純に 1 を仮定している．
・結果が変わらないことを確認したければ、試しに 1 から変えて計算してみればよい．
・市場価格は 1 だが，agent 価格は 1 とは限らない．agent価格とは経済主
　体が直面する価格のこと．

【注】
・以下のプログラムにおいて価格の初期値が至る所で現われるが、結局 1 を
　仮定しているため、ほとんどの部分で省略することができる (価格の初期値
　が現れる部分では結局 1 をかけたり、1 で割ったりしているだけであるので)．
・実際、多くのCGEモデルのコードでは価格の初期値を明示的に扱っていない
　ことが多い．
・しかし、以下ではモデルの構造をできる限り正確に表現するため、価格の初
　期値を明示的に表現している．

$offtext
parameter
    pq0(i)      i 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pf0(f)      f 要素のベンチマーク市場価格
    pg0         ベンチマークにおける政府支出財の価格
;
pq0(i) = 1;
pu0 = 1;
pf0(f) = 1;
pg0 = 1;
display pq0, pu0, pf0, pg0;

*       ----------------------------------------------------------------------
*       生産サイド
$ontext
ベンチマークの数量．

・SAM の金額データとベンチマーク価格より，ベンチマークの数量が導出でき
　る．
$offtext
parameter
    q0(ii,i)    i 部門の ii 財のベンチマーク生産量
    int0(ii,i)  i 部門の ii 財のベンチマーク投入量
    fd0(f,i)    i 部門の f 要素のベンチマーク投入量
;
*       数量 = 金額 / 価格
q0(i,ii)$(sam(i,ii) ge 0) = sam(i,ii) / pq0(i);
int0(i,ii)$(sam(i,ii) le 0) = - sam(i,ii) / pq0(i);
fd0(f,i) = - sam(f,i) / pf0(f);

display q0, int0, fd0;

*       ----------------------------------------------------------------------
*       消費サイド．
parameter
    u0          ベンチマークの効用水準
    cd0(i)      i 財のベンチマーク消費量
;
*       ベンチマークの効用，消費量
u0 = sam("u","u") / pu0;
cd0(i) = - sam(i,"u") / pq0(i);

display u0, cd0;

*       ----------------------------------------------------------------------
*       政府
parameter
    g0          ベンチマークにおける政府支出
    gd0(i)      ベンチマークにおける i 財に対する政府支出需要
;
*       ベンチマークの生産量，投入量
g0 = sam("pg","g") / pg0;
gd0(i) = - sam(i,"g") / pq0(i);

display g0, gd0;

*       ----------------------------------------------------------------------
*       要素賦存，所得
parameter
    e_f(f)      f 要素の賦存量
    inc0        ベンチマーク所得
;
*       ベンチマークの賦存量 (供給量)
e_f(f) = sam(f,"cons") / pf0(f);

*       ベンチマークの所得
inc0 = - sam("u","cons");

display e_f, inc0;

*       ----------------------------------------------------------------------
*       税金についてのパラメータ．
$ontext
・税は 3 種類．1) x 財に対する生産税，2) 労働課税，3 ) 一括税
・1 と 2 については従価税として扱う．
・従価税率はベンチマークデータより導出できる (税額 / タックスベース)．
・労働課税率は二つの部門で異なる値．

$offtext
parameter
    tf0(f,i)    ベンチマークにおける i 部門の f 要素に対する税率
    tq0(ii,i)   ベンチマークにおける i 部門の ii 財の生産に対する税率

    tf(f,i)     i 部門の f 要素に対する税率
    tq(ii,i)    i 部門の ii 財の生産に対する税率

    vlump0      ベンチマークにおける一括税
;
*       税率 = 税額 / タックスベース
tf0(f,i) =  0;
tf0("l",i) = sam("vtl",i) / sam("l",i);
tq0("x","x") = - sam("vtx","x") / sam("x","x");

display tf0, tq0;

tf(f,i) = tf0(f,i);
tq(ii,i) = tq0(ii,i);

*       一括税
vlump0 = sam("vtls","gov");
display vlump0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．

*       代替の弾力性：
parameter
    eta_q(i)    EOT: 生産物間の EOT
    sig_kl(i)   EOS: 部門 i における資本・労働の EOS
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
    sig_g       EOS: 政府支出における EOS               / 0.0 /
;
eta_q(i) = 4;
sig_kl(i) = 0.5;

display sig_kl, sig_u, sig_g;

$ontext
以下は要素賦存量と政府支出レベルを変更するシミュレーションをおこなうた
めのパラメータ．
$offtext
parameter
    s_f(f)      f 要素のスケールファクター
    s_lump      一括税のスケールファクター      / 1.0 /
    s_gov       政府支出のスケールファクター    / 1.0 /
;
s_f(f) = 1.0;
display s_f, s_lump, s_gov;

$ontext
政策の場合分け．

[fl_lump]
これが非ゼロの場合には，政府支出が一定となるように，税収を一括税で調整
する．

[fl_uni]
これが非ゼロの場合には，政府支出が一定となるように，税収を uniform な
労働課税で調整する．

$offtext
parameter
    fl_lump     非ゼロなら政府支出が一定となるように一括税で調整
    fl_uni      非ゼロなら Uniform な労働課税
;
fl_lump = yes;
fl_uni = no;

*       ======================================================================
$stitle モデルの記述
*       ======================================================================
$ontext
・同じモデルをMPSGEとMCPの二つの形式で記述する．
・可読性を高めるため、モデルの内生変数は大文字で表現している．
$offtext

*       ----------------------------------------------------------------------
$stitle MPSGE によるモデルの記述:
$ontext

*       モデル名
$model:ex_mpsge_mcp_01

*       アクティビティー変数 (sector) の宣言
$sectors:
    Q(i)        ! i 部門のアクティビティーレベル
    U           ! u 部門の生産量 (効用水準)
    G           ! g 部門の生産量 (政府支出水準)

*       価格変数 (commodity) の宣言
$commodities:
    PQ(i)       ! i 財の価格
    PU          ! u 財の価格 (効用の価格)
    PF(f)       ! f 要素の価格
    PG          ! 政府支出財の価格

*       所得変数 (consumer) の宣言
$consumers:
    CONS        ! 消費者の所得
    GOV         ! 政府の所得

*       補助変数の宣言
$auxiliary:
    TLUMP$fl_lump       ! 一括税
    TLUNI$fl_uni        ! 全ての部門に対して uniform な労働税率

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 i の生産関数
$prod:Q(i)  t:eta_q(i)  s:0  kl:sig_kl(i)
    o:PQ(ii)    q:q0(ii,i)      p:((1-tq0(ii,i))*pq0(ii))  a:GOV  t:tq(ii,i)
    i:PQ(ii)    q:int0(ii,i)    p:pq0(ii)
    i:PF(f)     q:fd0(f,i)      p:((1+tf0(f,i))*pf0(f))  kl:
+   a:GOV  t:tf(f,i)$(not fl_uni)  n:TLUNI$(fl_uni and lab(f))

*       部門 u の生産関数 (効用関数)
$prod:U  s:sig_u
    o:PU        q:u0
    i:PQ(i)     q:cd0(i)  p:pq0(i)

*       部門 g の生産関数 (政府支出)
$prod:G  s:sig_g
    o:PG        q:g0
    i:PQ(i)     q:gd0(i)  p:pq0(i)

*       ----------------------------------------------------------------------
*       予算制約

*       民間消費者の予算制約
$demand:CONS
    d:PU
    e:PF(f)     q:(e_f(f)*s_f(f))
    e:PG        q:(-vlump0*s_lump)  r:TLUMP$fl_lump

*       政府の予算制約
$demand:GOV
    d:PG
    e:PG        q:(vlump0*s_lump)  r:TLUMP$fl_lump

*       ----------------------------------------------------------------------
*       Constraints:

*       一括税は政府支出が一定となるように調整される．
$constraint:TLUMP$fl_lump
    G =e= s_gov;

*       労働税は政府支出が一定となるように調整される．
$constraint:TLUNI$fl_uni
    G =e= s_gov;

*       ----------------------------------------------------------------------
*       Report 用の変数
$report:
    v:Z_FD(f,i) i:PF(f) prod:Q(i)       ! i 部門の f 要素への需要

$offtext
*       preprocessor の呼び出し
$ontext
$sysinclude mpsgeset model_name

という書き方をする．
$offtext
$sysinclude mpsgeset ex_mpsge_mcp_01

*       変数の初期値
TLUMP.l = 1;
TLUNI.l = 0;

*       変数の初期値．
$ontext
pq0, pu0, pf0, pg0 は全て1と置いており，さらにMPSGEの価格変数の初期値
は1であるので，本来は設定する必要はないが，pq0~pg0を1としないケースに
も対応するため，以下のように初期値の設定をする．
$offtext
PQ.l(i) = pq0(i);
PU.l = pu0;
PF.l(f) = pf0(f);
PG.l = pg0;

*       Bounds:
$ontext
MPSGEでは変数のデフォールトの下限は 0．補助変数 tlump と tluni は負の値
もとる変数であるので，下限を変更しておく．
$offtext
TLUMP.lo = -inf;
TLUNI.lo = -inf;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data の下で均衡が成立しているかをチェックする．

これには iteration の回数をゼロとして解く．

model_name.iterlim = 0;

とすると iteration 回数を 0 に設定できる．
$offtext

ex_mpsge_mcp_01.iterlim = 0;
$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

$ontext
iteration 回数を増加して解く．

モデル，データが適切ならば，変数の初期値がそのまま解となるはず．
$offtext

ex_mpsge_mcp_01.iterlim = 10000;
$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

*       ----------------------------------------------------------------------
$stitle MCP 表現によるモデルの記述:
$ontext
・通常の数式表現によるモデルの記述．

$offtext
*       ----------------------------------------------------------------------
*       MCP によるモデルの記述
display "@ MCP によるモデルの記述";
$ontext
以下の手順に従う．

+ コスト，収入を表すパラメータを定義
+ シェアパラメータの計算
+ モデルを記述 (以下の順番)
  + 単位費用関数 & 価格指数の定義
  + ゼロ利潤条件
  + 需要関数の定義
  + 市場均衡条件
  + 所得の定義
  + その他の制約式

+ シェアパラメータを定義するのは，モデルを calibrated share form で記述
  するため．
+ MCPモデルでの変数には最後にアンダーバー (_) を付けている．
$offtext

*       以下はベンチマークの費用
parameter
    cost_q(i)   i 部門のコスト
    cost_va(i)  i 部門の付加価値
    rev_q(i)    i 部門の収入
    cost_u      効用
    cost_g      政府支出
;

*       i 部門の付加価値
cost_va(i) = sum(f, (1+tf0(f,i))*pf0(f)*fd0(f,i));
*       i 部門のコスト
cost_q(i) = cost_va(i) + sum(ii, pq0(ii)*int0(ii,i));
*       i 部門の収入
rev_q(i) = cost_q(i);

*       効用
cost_u = sum(i, pq0(i)*cd0(i));
*       政府支出
cost_g = sum(i, pq0(i)*gd0(i));

parameter
    sh_fd(f,i)          f 要素の投入シェア
    sh_va(i)            付加価値の投入シェア
    sh_int(ii,i)        中間財の投入シェア
    sh_q(ii,i)          生産における各財のシェア
    sh_cd(i)            消費における各財のシェア
    sh_gd(i)            政府支出における各財のシェア
    ;

*       f 要素の投入シェア
sh_fd(f,i)$cost_va(i) = (1+tf0(f,i))*pf0(f)*fd0(f,i) / cost_va(i);
*       付加価値の投入シェア
sh_va(i) = cost_va(i) / cost_q(i);
*       中間財の投入シェア
sh_int(ii,i) = pq0(ii)*int0(ii,i) / cost_q(i);

*       生産における各財のシェア
sh_q(ii,i) = (1-tq0(ii,i))*pq0(ii)*q0(ii,i) / rev_q(i);

*       消費における各財のシェア
sh_cd(i) = pq0(i)*cd0(i) / cost_u;
*       政府支出における各財のシェア
sh_gd(i) = pq0(i)*gd0(i) / cost_g;

display sh_fd, sh_va, sh_q, sh_int, sh_cd, sh_gd;

*       ----------------------------------------------------------------------
*       変数の宣言
$ontext
変数のタイプ

1) アクティビティー変数
2) 価格変数
3) 所得変数
4) 補助変数

5) 単位費用変数
6) 単位需要 (供給) 変数

1~4はMPSGEと同じ．5, 6 はMCPのみ定義する変数．

$offtext

*       アクティビティー変数 (sector) の宣言
variables
    Q_(i)       "i 部門のアクティビティーレベル"
    U_          "u 部門の生産量 (= 効用水準)"
    G_          "g 部門の生産量 (= 政府支出水準)"
;
*       価格変数 (commodity) の宣言
variables
    PQ_(i)      "i 財の価格"
    PU_         "u 財の価格 (= 効用の価格)"
    PF_(f)      "f 要素の価格"
    PG_         "政府支出財の価格"
;
*       所得変数 (consumer) の宣言
variables
    CONS_       "消費者の所得"
    GOV_        "政府の所得"
;
*       補助変数の宣言
variables
    TLUMP_      "一括税"
    TLUNI_      "全ての部門に対して uniform な労働税率"
;
*       単位費用変数，価格指数の宣言
variables
    C_Q_(i)     "i 部門の単位費用"
    C_U_        "u 部門の単位費用"
    C_G_        "g 部門の単位費用"

    P_OUT_(i)   "i 部門の生産の価格指数"
    P_VA_(i)    "i 部門の付加価値価格指数"
;
*       単位需要 (供給) 変数の宣言
variables
    A_INT_(ii,i)        "i 部門の ii 財への単位需要"
    A_F_(f,i)           "i 部門の f 要素への単位需要"
    A_Q_(ii,i)          "i 部門の ii 財の単位供給"
    A_C_(i)             "i 財への単位消費需要"
    A_G_(i)             "i 財への単位政府支出需要"
;
*       ----------------------------------------------------------------------
*       式の宣言
$ontext
式の名前

+ MCPでモデルを定義するときには，式と変数の対応関係を指定する必要があ
　る (model 命令の部分を参照)．
+ 式と変数の対応関係を把握しやすいように
  + A という変数に対応する式 → e_A
  + B という変数に対応する式 → e_B
  というような名前の付け方を採用している．

式の意味

各変数に対する各式の意味は以下の通り

1) アクティビティー変数 →      利潤最大化条件 (ゼロ利潤条件)
2) 価格変数             →      市場均衡条件 or 価格指数の定義式
3) 所得変数             →      所得の定義式
4) 補助変数             →      制約式
5) 単位費用変数         →      単位費用の定義式
6) 単位需要 (供給) 変数 →      単位需要 (供給) の定義式

$offtext
*       アクティビティー変数 (sector) の宣言
equations
    e_Q_(i)     "i 部門のアクティビティーレベル"
    e_U_        "u 部門の生産量 (= 効用水準)"
    e_G_        "g 部門の生産量 (= 政府支出水準)"
;
*       価格変数 (commodity) の宣言
equations
    e_PQ_(i)    "i 財の価格"
    e_PU_       "u 財の価格 (= 効用の価格)"
    e_PF_(f)    "f 要素の価格"
    e_PG_       "政府支出財の価格"
;
*       所得変数 (consumer) の宣言
equations
    e_CONS_     "消費者の所得"
    e_GOV_      "政府の所得"
;
*       補助変数の宣言
equations
    e_TLUMP_    "一括税"
    e_TLUNI_    "全ての部門に対して uniform な労働税率"
;
*       単位費用変数，価格指数の宣言
equations
    e_C_Q_(i)   "i 部門の単位費用"
    e_C_U_      "u 部門の単位費用"
    e_C_G_      "g 部門の単位費用"

    e_P_OUT_(i) "i 部門の生産の価格指数"
    e_P_VA_(i)  "i 部門の付加価値価格指数"
;
*       単位需要 (供給) 変数の宣言
equations
    e_A_INT_(ii,i)      "i 部門の ii 財への単位需要"
    e_A_F_(f,i)         "i 部門の f 要素への単位需要"
    e_A_Q_(ii,i)        "i 部門の ii 財の単位供給"
    e_A_C_(i)           "i 財への単位消費需要"
    e_A_G_(i)           "i 財への単位政府支出需要"
;
*       ----------------------------------------------------------------------
*       式の定義
*       ----------------------------------------------------------------------

*       ----------------------------------------------------------------------
*       単位費用関数 & 価格指数

*       i 部門の生産の価格指数
e_P_OUT_(i) ..

    P_OUT_(i) =e=
    (sum(ii$q0(ii,i),
        sh_q(ii,i)*((1-tq(ii,i))*PQ_(ii)/((1-tq0(ii,i))*pq0(ii)))**(1+eta_q(i))
    ))**(1/(1+eta_q(i)));

*       i 部門の付加価値価格指数
e_P_VA_(i) ..

    P_VA_(i) =e=
    (sum(f$fd0(f,i),
        sh_fd(f,i)*((1+(tf(f,i)$(not fl_uni) + TLUNI_$(fl_uni and lab(f))))*PF_(f)
            /((1+tf0(f,i))*pf0(f)))**(1-sig_kl(i))
    )**(1/(1-sig_kl(i))))$(sig_kl(i) ne 1)
    + (prod(f$fd0(f,i),
        ((1+(tf(f,i)$(not fl_uni) + TLUNI_$(fl_uni and lab(f))))*PF_(f)
            /((1+tf0(f,i))*pf0(f)))**sh_fd(f,i)))$(sig_kl(i) eq 1);

*       i 部門の単位費用
e_C_Q_(i) ..

    C_Q_(i) =e=
    sum(ii$int0(ii,i), sh_int(ii,i)*PQ_(ii)/pq0(ii)) + sh_va(i)*P_VA_(i);

*       u 部門の単位費用
e_C_U_ ..

    C_U_ =e=
    (sum(i$cd0(i), sh_cd(i)*(PQ_(i)/pq0(i))**(1-sig_u))**(1/(1-sig_u)))$(sig_u ne 1)
    + (prod(i$cd0(i), (PQ_(i)/pq0(i))**(sh_cd(i))))$(sig_u eq 1);

*       g 部門の単位費用
e_C_G_ ..

    C_G_ =e=
    (sum(i$gd0(i), sh_gd(i)*(PQ_(i)/pq0(i))**(1-sig_g))**(1/(1-sig_g)))$(sig_g ne 1)
    + (prod(i$gd0(i), (PQ_(i)/pq0(i))**(sh_gd(i))))$(sig_g eq 1);

*       ----------------------------------------------------------------------
*       ゼロ利潤条件 (利潤最大化条件)
$ontext
ゼロ利潤条件 → 単位費用 = 価格
$offtext

*       i 部門のアクティビティーレベル
e_Q_(i) .. c_q_(i) =e= p_out_(i);

*       u 部門の生産量 (効用水準)
e_U_ .. C_U_ =e= PU_;

*       g 部門の生産量 (政府支出水準)
e_G_ .. C_G_ =e= PG_;

*       ----------------------------------------------------------------------
*       需要・供給関数

*       i 部門の ii 財への単位需要
e_A_INT_(ii,i)$int0(ii,i) .. A_INT_(ii,i) =e= int0(ii,i);

*       i 部門の f 要素への単位需要
e_A_F_(f,i)$fd0(f,i) ..

    A_F_(f,i)
    =e= fd0(f,i)
    * (P_VA_(i) /
        ((1+(tf(f,i)$(not fl_uni) + tluni_$(fl_uni and lab(f))))*PF_(f)
            /((1+tf0(f,i))*pf0(f)))
    )**sig_kl(i);

*       i 部門の ii 財の単位供給
e_A_Q_(ii,i)$q0(ii,i) ..

    A_Q_(ii,i)
    =e= q0(ii,i) * (P_OUT_(i) / ((1-tq(ii,i))*PQ_(ii)/((1-tq0(ii,i))*pq0(ii)))
    )**(-eta_q(i));

*       i 財への単位消費需要
e_A_C_(i)$cd0(i) ..

    A_C_(i) =e= cd0(i) * (C_U_/(PQ_(i)/pq0(i)))**(sig_u);

*       i 財への単位政府支出需要
e_A_G_(i) ..

    A_G_(i) =e= gd0(i) * (C_G_/(PQ_(i)/pq0(i)))**(sig_g);

*       ----------------------------------------------------------------------
*       市場均衡条件
$ontext
市場均衡条件 → 供給 = 需要
$offtext

*       i 財の価格
e_PQ_(i) ..

    sum(ii$q0(i,ii), A_Q_(i,ii)*Q_(ii)) =e=
    sum(ii$int0(i,ii), A_INT_(i,ii)*Q_(ii))
    + A_C_(i)*U_ + A_G_(i)*G_;

*       f 要素の価格
e_PF_(f) ..

    e_f(f)*s_f(f) =e= sum(i$fd0(f,i), A_F_(f,i)*Q_(i));

*       u 財の価格 (効用の価格)
e_PU_ ..

    u0*U_ =e= CONS_ / PU_;

*       政府支出財の価格
e_PG_ ..

    g0*G_ =e= GOV_ / PG_;

*       ----------------------------------------------------------------------
*       所得の定義:

*       消費者の所得
e_CONS_ ..

    CONS_ =e= sum(f, PF_(f)*e_f(f)*s_f(f))
    - PG_*vlump0*s_lump*(1$(not fl_lump) + TLUMP_$fl_lump);

*       政府の所得
e_GOV_ ..

    GOV_ =e=
    PG_*vlump0*s_lump*(1$(not fl_lump) + TLUMP_$fl_lump)
    + sum((i,ii)$q0(ii,i), tq(ii,i)*PQ_(ii)*A_Q_(ii,i)*Q_(i))
    + sum((f,i)$fd0(f,i),
        (tf(f,i)$(not fl_uni) + TLUNI_$(fl_uni and lab(f)))*PF_(f)*A_F_(f,i)*Q_(i));

*       ----------------------------------------------------------------------
*       補助変数に対する制約式

*       一括税
e_TLUMP_$fl_lump .. G_ =e= s_gov;

*       全ての部門に対して uniform な労働税率
e_TLUNI_$fl_uni .. G_ =e= s_gov;

*       ----------------------------------------------------------------------
*       モデルの宣言

model ex_mcp_01 Model written in MCP formula /

                e_P_OUT_.P_OUT_, e_P_VA_.P_VA_, e_C_Q_.C_Q_,
                e_C_U_.C_U_, e_C_G_.C_G_,

                e_Q_.Q_, e_U_.U_, e_G_.G_, e_A_INT_.A_INT_, e_A_F_.A_F_,
		e_A_Q_.A_Q_,
                e_A_C_.A_C_, e_A_G_.A_G_,

                e_PQ_.PQ_, e_PF_.PF_, e_PU_.PU_, e_PG_.PG_,

                e_CONS_.CONS_, e_GOV_.GOV_,

                e_TLUMP_.TLUMP_, e_TLUNI_.TLUNI_ /;

*       変数の下限の設定
$ontext
+ アクティビティー変数 & 単位需要変数 → 0 を設定
+ 価格変数 → 1e-6 を設定

+ 価格変数に 1e-6 を設定するのは "division by zero error" を避けるため．
$offtext
Q_.lo(i) = 0;
U_.lo = 0;
G_.lo = 0;
PQ_.lo(i) = 1e-6;
PU_.lo = 1e-6;
PF_.lo(f) = 1e-6;
PG_.lo = 1e-6;
CONS_.lo = 0;
GOV_.lo = 0;
* TLUMP_.lo = 0;
* TLUNI_.lo = 0;
C_Q_.lo(i) = 1e-6;
C_U_.lo = 1e-6;
C_G_.lo = 1e-6;
P_OUT_.lo(i) = 1e-6;
P_VA_.lo(i) = 1e-6;
A_INT_.lo(ii,i) = 0;
A_F_.lo(f,i) = 0;
A_Q_.lo(ii,i) = 0;
A_C_.lo(i) = 0;
A_G_.lo(i) = 0;

*       変数の初期値の設定
Q_.l(i) = 1;
U_.l = 1;
G_.l = 1;
PQ_.l(i) = pq0(i);
PU_.l = pu0;
PF_.l(f) = pf0(f);
PG_.l = pg0;
CONS_.l = u0;
GOV_.l = g0;
TLUMP_.l = 1;
TLUNI_.l = 0;
C_Q_.l(i) = 1;
C_U_.l = 1;
C_G_.l = 1;
P_OUT_.l(i) = 1;
P_VA_.l(i) = 1;
A_INT_.l(ii,i) = int0(ii,i);
A_F_.l(f,i) = fd0(f,i);
A_Q_.l(ii,i) = q0(ii,i);
A_C_.l(i) = cd0(i);
A_G_.l(i) = gd0(i);

*       名目値の規準化
CONS_.fx = cons_.l;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
MPSGEのケースと同様 Benchmark replication & Cleanup calculation でチェック．
$offtext
ex_mcp_01.iterlim = 0;
solve ex_mcp_01 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";
ex_mcp_01.iterlim = 10000;
solve ex_mcp_01 using mcp;

*       ======================================================================
$stitle シミュレーション
*       ======================================================================
display "@ シミュレーション";

*       結果表示用の index の定義．
set     pol     / x, y, z, u, g, px, py, pk, pl /
        case    Cases           /
                b       ベンチマーク均衡
                c1      賦存量の比例的増加
                c2      労働課税率を2倍に上昇させる
                c3      全ての部門に対して uniform な労働税率に変更
                c4      一括税以外の税を撤廃
                /
;
display case;

*       結果表示用のパラメータ
parameter
    res_l               MPSGE
    res_l_              MCP
    chk_results         Compare results from MPSGE and MCP models
;
*       結果保存用のマクロ（mpsge用）
$macro res_mpsge(case) \
    res_l("qx",case) = Q.l("x");\
    res_l("qy",case) = Q.l("y");\
    res_l("qz",case) = Q.l("z");\
    res_l("u",case) = U.l;\
    res_l("g",case) = G.l;\
    res_l("px",case) = PQ.l("x") / PU.l;\
    res_l("py",case) = PQ.l("y") / PU.l;\
    res_l("pz",case) = PQ.l("z") / PU.l;\
    res_l("pk",case) = PF.l("k") / PU.l;\
    res_l("pl",case) = PF.l("l") / PU.l;

*       結果保存用のマクロ（mcp用）
$macro res_mcp(case) \
    res_l_("qx",case) = Q_.l("x");\
    res_l_("qy",case) = Q_.l("y");\
    res_l_("qz",case) = Q_.l("z");\
    res_l_("u",case) = U_.l;\
    res_l_("g",case) = G_.l;\
    res_l_("px",case) = PQ_.l("x") / PU_.l;\
    res_l_("py",case) = PQ_.l("y") / PU_.l;\
    res_l_("pz",case) = PQ_.l("z") / PU_.l;\
    res_l_("pk",case) = PF_.l("k") / PU_.l;\
    res_l_("pl",case) = PF_.l("l") / PU_.l;

*       ======================================================================
*       MPSGEのモデルによるシミュレーション
display "@ MPSGEのモデルによるシミュレーション";
$ontext
まずはベンチマークの均衡．

ベンチマークの変数値．
"b" はベンチマークの値を表す．
$offtext

$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

*       Macro実行
res_mpsge("b");

*       ----------------------------------------------------------------------
*       C1: 賦存量の比例的増加．
display "@ C1: 賦存量の比例的増加．";
$ontext
賦存量にあたる外生変数 (パラメータ) を比例的 (10%) に増加させるショック．

[Check]
・生産量，効用等の実質値はどう変化するか？
・価格 (名目変数) はどう変化するか？
$offtext

*       10% ずつの増加．
s_f(f) = 1.1;
s_lump = 1.1;
s_gov = 1.1;

$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

s_f(f) = 1;
s_lump = 1;
s_gov = 1;

*       結果をパラメータに代入．
res_mpsge("c1");

*       ----------------------------------------------------------------------
*       C2: 労働課税率を2倍に上昇させる．
display "@ C2: 労働課税率を2倍に上昇させる．";
$ontext
既存の労働課税率を 2 倍に上昇させるショック．

[Check]
・消費者の効用はどう変化するか？
・元々，労働課税率は x 部門が高い．各部門の生産量にどのような影響があるか？
$offtext

tf("l",i) = tf0("l",i) * 2;

$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

tf(f,i) = tf0(f,i);

res_mpsge("c2");

*       ----------------------------------------------------------------------
*       C3: 全ての部門に対して uniform な労働税率に変更．
display "@ C3: 全ての部門に対して uniform な労働税率に変更．";
$ontext
ベンチマークでは各部門における労働課税率は異なっている．これを全ての部
門に対し一様な税率に変更するショック．

一様な税率の水準は政府支出が一定となるように決定する．

[Check]
・効用はどう変化するか？改善するか？
$offtext

fl_lump = no;
fl_uni = yes;

$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

fl_lump = yes;
fl_uni = no;

res_mpsge("c3");

*       ----------------------------------------------------------------------
*       C4: 一括税以外の税を撤廃．
display "@ C4: 一括税以外の税を撤廃．";
$ontext
一括税以外の税を撤廃する (税率をゼロにする) ショック．

一括税の額は政府支出が一定となるように調整する．

[Check]
・効用はどう変化するか？改善するか？
$offtext

tq(ii,i) = 0;
tf(f,i) = 0;

$include ex_mpsge_mcp_01.gen
solve ex_mpsge_mcp_01 using mcp;

tq(ii,i) = tq0(ii,i);
tf(f,i) = tf0(f,i);

res_mpsge("c4");

*       ======================================================================
*       MCPのモデルによるシミュレーション
display "@ MCPのモデルによるシミュレーション";

solve ex_mcp_01 using mcp;

res_mcp("b");

*       ----------------------------------------------------------------------
*       C1: 賦存量の比例的増加．
display "@ C1: 賦存量の比例的増加．";
$ontext

賦存量にあたる外生変数 (パラメータ) を比例的 (10%) に増加させるショッ
ク．

$offtext

*       10% ずつの増加．
s_f(f) = 1.1;
s_lump = 1.1;
s_gov = 1.1;

solve ex_mcp_01 using mcp;

s_f(f) = 1;
s_lump = 1;
s_gov = 1;

*       結果をパラメータに代入．
res_mcp("c1");

*       ----------------------------------------------------------------------
*       C2: 労働課税率を2倍に上昇させる．
display "@ C2: 労働課税率を2倍に上昇させる．";
$ontext
既存の労働課税率を 2 倍に上昇させるショック．

$offtext

tf("l",i) = tf0("l",i) * 2;

solve ex_mcp_01 using mcp;

tf(f,i) = tf0(f,i);

res_mcp("c2");

*       ----------------------------------------------------------------------
*       C3: 全ての部門に対して uniform な労働税率に変更．
display "@ C3: 全ての部門に対して uniform な労働税率に変更．";
$ontext
ベンチマークでは各部門における労働課税率は異なっている．これを全ての部
門に対し一様な税率に変更するショック．

一様な税率の水準は政府支出が一定となるように決定する．

$offtext

fl_lump = no;
fl_uni = yes;

solve ex_mcp_01 using mcp;

fl_lump = yes;
fl_uni = no;

res_mcp("c3");

*       ----------------------------------------------------------------------
*       C4: 一括税以外の税を撤廃．
display "@ C4: 一括税以外の税を撤廃．";
$ontext
一括税以外の税を撤廃する (税率をゼロにする) ショック．

一括税の額は政府支出が一定となるように調整する．

$offtext

tq(ii,i) = 0;
tf(f,i) = 0;

solve ex_mcp_01 using mcp;

tq(ii,i) = tq0(ii,i);
tf(f,i) = tf0(f,i);

res_mcp("c4");

*       ----------------------------------------------------------------------
*       変数の変化率の導出．
$ontext
ベンチマークからの変数の変化率を導出．

+ MPSGEモデルとMCPのモデルが同じ結果を導くことを確認する．

$offtext
chk_results("mpsge",pol,case)$res_l(pol,"b")
    = round(100 * (res_l(pol,case) / res_l(pol,"b") - 1), 6);
chk_results("mcp",pol,case)$res_l_(pol,"b")
    = round(100 * (res_l_(pol,case) / res_l_(pol,"b") - 1), 6);

*       結果の表示．
display res_l, res_l_;
display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 70
* coding: utf-8
* End:
