$title ex_prod_03.gms: 生産関数が3段階CES型のケース
display "@ ex_prod_03.gms: 生産関数が3段階CES型のケース";
$ontext
Time-stamp:     <2022-01-29 11:20:04 st>
First-written:  <2010/03/12>

□ モデル
・生産関数が 3 段階の CES 型関数であるケース．
・生産要素に天然資源をプラス．


□ SAM
           |         Sectors         |  Consumers
Commoditiy |          (部門)         |   (消費者)
   (財)    |    x       y        u   |    cons
-----------------------------------------------------
       px  |  130             -130   |
       py  |  -20     120     -100   |
       pu  |                   230   |    -230
       pl  |  -25     -85            |     110
       pk  |  -75     -35            |     110
       pr  |  -10                    |      10
-----------------------------------------------------

* pr は天然資源の行


□ 部門 x の生産関数

         x
        / \ <- sig_xx
       /   \
      /     \
     y     / \ <- sig_x
          /   \
         /     \
        l     / \ <- sig_kr
             /   \
            /     \
           k       r

・3段階の CES 型関数．
・投入物は，中間財 (y財)，資本，労働，天然資源．
・代替の弾力性
　+ トップレベルの代替の弾力性は sig_xx，
　+ 労働と資本・天然資源の間の代替の弾力性は sig_x，
　+ 資本と天然資源の間の代替の弾力性は sig_kr

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
              x      y       u    cons
px          130      0    -130       0
py          -20    120    -100       0
pu            0      0     230    -230
pl          -25    -85       0     110
pk          -75    -35       0     110
pr          -10      0       0      10
;
display sam;

parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
    pr0         天然資源のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
pr0 = 1;
display px0, py0, pu0, pk0, pl0, pr0;

parameter
    x0          x 財のベンチマーク生産量
    y0          y 財のベンチマーク生産量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
    yx0         x 部門における y 財のベンチマーク投入量
    rx0         x 部門における天然資源のベンチマーク投入量
;

x0 = sam("px","x") / px0;
y0 = sam("py","y") / py0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;

yx0 = - sam("py","x") / py0;
rx0 = - sam("pr","x") / pr0;

display x0, y0, kx0, ky0, lx0, ly0, yx0, rx0;

parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
;
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;

display u0, cx0, cy0;

parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    e_r         天然資源の賦存量
    inc0        ベンチマーク所得
;
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;
e_r = sam("pr","cons") / pr0;
inc0 = - sam("pu","cons");

display e_l, e_k, e_r, inc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

parameter
    sig_x       EOS: 部門 x の l & K-R の EOS           / 0.5 /
    sig_xx      EOS: 部門 x の VA & y の EOS            / 0.1 /
    sig_kr      EOS: 部門 x の K & R の EOS             / 0.1 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
;
display sig_x, sig_y, sig_xx, sig_kr;

parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
;
display s_l, s_k;

$ontext
MPSGEモデル

$offtext

* option sysout = on;

$ontext
$model:ex_prod_03

*       MPSGE のデバッグオプション (sysout = on にして利用する)
* $funlog:.true.

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量
    u           ! u 部門の生産量 (効用水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格
    pr          ! 天然資源の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:x  s:sig_xx  va:sig_x  kr(va):sig_kr
    o:px        q:x0
    i:pl        q:lx0  p:pl0  va:
    i:pk        q:kx0  p:pk0  kr:
    i:py        q:yx0  p:py0
    i:pr        q:rx0  p:pr0  kr:

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0

*       ----------------------------------------------------------------------
*       予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)
    e:pr        q:(e_r)

$offtext

$sysinclude mpsgeset ex_prod_03

*       結果表示用のパラメータ
parameter
    chk_results
;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data が 均衡となっているかをチェックする．
$offtext

ex_prod_03.iterlim = 0;
$include ex_prod_03.gen
solve ex_prod_03 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

ex_prod_03.iterlim = 10000;
$include ex_prod_03.gen
solve ex_prod_03 using mcp;

*       "b" はベンチマークの値を表す．
chk_results("x","b") = x.l;
chk_results("y","b") = y.l;
chk_results("u","b") = u.l;
chk_results("px","b") = px.l / pu.l;
chk_results("py","b") = py.l / pu.l;
chk_results("pk","b") = pk.l / pu.l;
chk_results("pl","b") = pl.l / pu.l;

*       ----------------------------------------------------------------------
*       労働賦存量の 10% の増加:
display "@ 労働賦存量の 10% の増加:";

s_l = 1.1;

$include ex_prod_03.gen
solve ex_prod_03 using mcp;

chk_results("x","cf") = x.l;
chk_results("y","cf") = y.l;
chk_results("u","cf") = u.l;
chk_results("px","cf") = px.l / pu.l;
chk_results("py","cf") = py.l / pu.l;
chk_results("pk","cf") = pk.l / pu.l;
chk_results("pl","cf") = pl.l / pu.l;

display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 80
* coding: utf-8
* End:
