$title ex_prod_04.gms: 結合生産があるケース
display "@ ex_prod_04.gms: 結合生産があるケース";
$ontext
Time-stamp:     <2023-05-01 15:41:40 st>
First-written:  <2010/03/12>

□ モデル
・結合生産があるケース．
・x 部門が y 財も生産する．

□ SAM (ベンチマークデータ)
           |         Sectors         |  Consumers
Commoditiy |          (部門)         |   (消費者)
   (財)    |    x       y        u   |    cons
-----------------------------------------------------
       px  |  100             -100   |
       py  |   20     100     -120   |
       pu  |                   220   |    -220
       pl  |  -35     -75            |     110
       pk  |  -85     -25            |     110
-----------------------------------------------------

□ 部門 x の生産関数

     xx      xy
      \     /
       \   /
        \ / <- eta_x
         |
        / \ <- sig_x
       /   \
      /     \
     k      l

・投入側は ex_prod_1.gms と同じ．
・産出側
　・x 部門が x 財だけではなく，y 財も生産する．
　・xx が x の生産量，xy が y の生産量．
　・xx と xy の配分は CET (constant elastiticty of transformation) 関数に従っておこなわれる．
　・xx と xy の間の EOT (elastiticty of transformation) は eta_x．

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
              x      y       u    cons
px          100      0    -100       0
py           20    100    -120       0
pu            0      0     220    -220
pl          -35    -75       0     110
pk          -85    -25       0     110
;
display sam;

parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
display px0, py0, pu0, pk0, pl0;

parameter
    xx0         x 部門の x 財のベンチマーク生産量
    xy0         x 部門の y 財のベンチマーク生産量
    y0          y 財のベンチマーク生産量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
;

xx0 = sam("px","x") / px0;
xy0 = sam("py","x") / py0;
y0 = sam("py","y") / py0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;

display xx0, xy0, y0, kx0, ky0, lx0, ly0;

parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
;
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;

display u0, cx0, cy0;

parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    inc0        ベンチマーク所得
;
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;
inc0 = - sam("pu","cons");

display e_l, e_k, inc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

parameter
    sig_x       EOS: 部門 x における資本・労働の EOS    / 0.5 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
    eta_x       EOT: x 部門における EOT                 / 3.0 /
;
display sig_x, sig_y, eta_x;

parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
;
display s_l, s_k;

$ontext
MPSGEモデル

$offtext

$ontext
$model:ex_prod_04

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量
    u           ! u 部門の生産量 (効用水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数

*       変数 x は x 部門の activity のレベル．生産量ではない．x 部門による x 財，y
*       財の生産量は $report コマンドを利用して出力する．
$prod:x  s:sig_x  t:eta_x
    o:px        q:xx0  p:px0
    o:py        q:xy0  p:py0
    i:pk        q:kx0  p:pk0
    i:pl        q:lx0  p:pl0

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0

*       ----------------------------------------------------------------------
*       予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)

*       x 部門の生産量の表示．
$report:
    v:q_xx      o:px    prod:x  ! x 部門による x 財の生産量
    v:q_xy      o:py    prod:x  ! x 部門による y 財の生産量

$offtext

$sysinclude mpsgeset ex_prod_04

*       結果表示用のパラメータ
parameter
    chk_results
;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data が 均衡となっているかをチェックする．
$offtext

ex_prod_04.iterlim = 0;
$include ex_prod_04.gen
solve ex_prod_04 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

ex_prod_04.iterlim = 10000;
$include ex_prod_04.gen
solve ex_prod_04 using mcp;

*       "b" はベンチマークの値を表す．
chk_results("x","b") = x.l;
chk_results("y","b") = y.l;
chk_results("u","b") = u.l;
chk_results("px","b") = px.l / pu.l;
chk_results("py","b") = py.l / pu.l;
chk_results("pk","b") = pk.l / pu.l;
chk_results("pl","b") = pl.l / pu.l;

*       ----------------------------------------------------------------------
*       労働賦存量の 10% の増加:
display "@ 労働賦存量の 10% の増加:";

s_l = 1.1;

$include ex_prod_04.gen
solve ex_prod_04 using mcp;

chk_results("x","cf") = x.l;
chk_results("y","cf") = y.l;
chk_results("u","cf") = u.l;
chk_results("px","cf") = px.l / pu.l;
chk_results("py","cf") = py.l / pu.l;
chk_results("pk","cf") = pk.l / pu.l;
chk_results("pl","cf") = pl.l / pu.l;

display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 80
* coding: utf-8
* End:
