$title ex_prod_07.gms: 生産物に税がかかるケース．
display "@ ex_prod_07.gms: 生産物に税がかかるケース．";
$ontext
Time-stamp:     <2023-05-01 15:50:23 st>
First-written:  <2010/03/12>

□ モデル
・ex_prod_4.gms + x部門の生産物に税金がかるケース．

□ SAM (ベンチマークデータ)
           |         Sectors         |  Consumers 
Commoditiy |          (部門)         |   (消費者)
   (財)    |    x       y        u   |    cons
---------------------------------------------------
       px  |  120             -120   |
       py  |   25     100     -125   |
       pu  |                   245   |    -245
       pl  |  -35     -75            |     110
       pk  |  -85     -25            |     110
      vtx  |  -20                    |      20
      vty  |   -5                    |       5
---------------------------------------------------

□ 税金
・ベンチマークにおいてx部門の生産 (x財とy財) に税金がかかっているケース．
・vtx と vty が税金を表す行．
・120 と 25 という x 部門の生産額は市場価格表示．つまり，

　120 = 100 (生産者価格表示の生産額) + 20 (生産物税額)
　25 = 20 (生産者価格表示の生産額) + 5 (生産物税額)

・生産物に対する税は従価税として扱う．
　さらに，税率はグロスの形で表す．つまり，

  生産者価格 = (1 - t) * 市場価格
  t = 生産者価格 = (1 - t) * 市場価格


　という形式で税率 t を表現する．
・税収は直接消費者に lump-sum で還元される．


□ 部門 x の生産関数

     xx      xy
      \     /
       \   /
        \ / <- eta_x
         |
        / \ <- sig_x
       /   \
      /     \
     k      l

・投入側は ex_prod_1.gms と同じ．
・産出側
　・x 部門が x 財だけではなく，y 財も生産する．
　・xx が x の生産量，xy が y の生産量．
　・xx と xy の配分は CET (constant elastiticty of transformation) 関数に従っておこなわれる．
　・xx と xy の間の EOT (elastiticty of transformation) は eta_x．

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
              x      y       u    cons
px          120      0    -120       0
py           25    100    -125       0
pu            0      0     245    -245
pl          -35    -75       0     110
pk          -85    -25       0     110
vtx         -20      0       0      20
vty          -5      0       0       5
;
display sam;

parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
display px0, py0, pu0, pk0, pl0;

parameter
    xx0         x 部門の x 財のベンチマーク生産量
    xy0         x 部門の y 財のベンチマーク生産量
    y0          y 財のベンチマーク生産量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
;

xx0 = sam("px","x") / px0;
xy0 = sam("py","x") / py0;
y0 = sam("py","y") / py0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;

display xx0, xy0, y0, kx0, ky0, lx0, ly0;

parameter
    txx0        ベンチマークにおける x 部門の x 財に対する税率
    txy0        ベンチマークにおける x 部門の y 財に対する税率
    txx         x 部門の x 財に対する税率
    txy         x 部門の y 財に対する税率
;
txx0 = - sam("vtx","x") / sam("px","x");
txy0 = - sam("vty","x") / sam("py","x");

display txx0, txy0;

txx = txx0;
txy = txy0;
display txx, txy;

parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
;
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;

display u0, cx0, cy0;

parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    inc0        ベンチマーク所得
;
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;
inc0 = - sam("pu","cons");

display e_l, e_k, inc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

parameter
    sig_x       EOS: 部門 x における資本・労働の EOS    / 0.5 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
    eta_x       EOT: x 部門における EOT                 / 3.0 /
;
display sig_x, sig_y, eta_x;

parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
;
display s_l, s_k;

$ontext
MPSGEモデル

$offtext

$ontext
$model:ex_prod_07

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量
    u           ! u 部門の生産量 (効用水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数

*       変数 x は x 部門の activity のレベル．生産量ではない．x 部門による x 財，y
*       財の生産量は $report コマンドを利用して出力する．
$prod:x  s:sig_x  t:eta_x
    o:px        q:xx0  p:((1-txx0)*px0)  a:cons  t:txx
    o:py        q:xy0  p:((1-txy0)*py0)  a:cons  t:txy
    i:pk        q:kx0  p:pk0
    i:pl        q:lx0  p:pl0

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0

*       ----------------------------------------------------------------------
*       予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)

*       x 部門の生産量の表示．
$report:
    v:q_xx      o:px    prod:x  ! x 部門による x 財の生産量
    v:q_xy      o:py    prod:x  ! x 部門による y 財の生産量

$offtext

$sysinclude mpsgeset ex_prod_07

*       結果表示用のパラメータ
parameter
    chk_results
;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data が 均衡となっているかをチェックする．
$offtext

ex_prod_07.iterlim = 0;
$include ex_prod_07.gen
solve ex_prod_07 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

ex_prod_07.iterlim = 10000;
$include ex_prod_07.gen
solve ex_prod_07 using mcp;

*       "b" はベンチマークの値を表す．
chk_results("x","b") = x.l;
chk_results("y","b") = y.l;
chk_results("u","b") = u.l;
chk_results("px","b") = px.l / pu.l;
chk_results("py","b") = py.l / pu.l;
chk_results("pk","b") = pk.l / pu.l;
chk_results("pl","b") = pl.l / pu.l;

*       ----------------------------------------------------------------------
*       Decrease in tax rates on sector x by 50%:
display "@ Decrease in tax rates on sector x by 50%:";
$ontext
x部門の生産物に対する税を半分の水準にする．

Q:
・生産はどう変化するか？
・効用はどう変化するか？
$offtext

txx = txx*0.5;
txy = txy*0.5;

$include ex_prod_07.gen
solve ex_prod_07 using mcp;

chk_results("x","cf") = x.l;
chk_results("y","cf") = y.l;
chk_results("u","cf") = u.l;
chk_results("px","cf") = px.l / pu.l;
chk_results("py","cf") = py.l / pu.l;
chk_results("pk","cf") = pk.l / pu.l;
chk_results("pl","cf") = pl.l / pu.l;

display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 80
* coding: utf-8
* End:
