$title ex_tech_01.gms: MPSGE における技術進歩の導入方法
display "@ ex_tech_01.gms: MPSGE における技術進歩の導入方法";
$ontext
Time-stamp:     <2022-01-29 11:19:38 st>
First-written:  <2010/03/12>

このプログラムは基本的に「ex_prod_01.gms」をベースにしている。

以下が生産性を表すパラメータ

+ lambda: x 部門の全要素生産性
+ lambda_k: x 部門の資本投入についての生産性パラメータ
+ lambda_l: x 部門の労働投入についての生産性パラメータ

どれもベンチマークでは1の値


□ SAM (ベンチマークデータ)
           |         Sectors         |  Consumers
Commoditiy |          (部門)         |   (消費者)
   (財)    |    x       y        u   |    cons
-----------------------------------------------------
       px  |  100             -100   |
       py  |          100     -100   |
       pu  |                   200   |    -200
       pl  |  -25     -75            |     100
       pk  |  -75     -25            |     100
-----------------------------------------------------


□ 部門 x の生産関数

         x
        / \ <- sig_x
       /   \
      /     \
     k      l

・CES 型関数．
・投入物は資本，労働のみ．
・代替の弾力性は sig_x．

$offtext

*       ----------------------------------------------------------------------
*       ベンチマーク・データ:
display "@ ベンチマーク・データ:";

table   sam     ベンチマークデータ
               x       y        u     cons
px           100       0     -100        0
py             0     100     -100        0
pu             0       0      200     -200
pl           -25     -75        0      100
pk           -75     -25        0      100
;
display sam;

parameter
    px0         x 財のベンチマーク市場価格
    py0         y 財のベンチマーク市場価格
    pu0         u 財のベンチマーク市場価格
    pk0         資本のベンチマーク市場価格
    pl0         労働のベンチマーク市場価格
;

px0 = 1;
py0 = 1;
pu0 = 1;
pk0 = 1;
pl0 = 1;
display px0, py0, pu0, pk0, pl0;

parameter
    x0          x 財のベンチマーク生産量
    y0          y 財のベンチマーク生産量
    kx0         x 部門における資本のベンチマーク投入量
    ky0         y 部門における資本のベンチマーク投入量
    lx0         x 部門における労働のベンチマーク投入量
    ly0         y 部門における労働のベンチマーク投入量
;

x0 = sam("px","x") / px0;
y0 = sam("py","y") / py0;
kx0 = - sam("pk","x") / pk0;
ky0 = - sam("pk","y") / pk0;
lx0 = - sam("pl","x") / pl0;
ly0 = - sam("pl","y") / pl0;

display x0, y0, kx0, ky0, lx0, ly0;

parameter
    u0          ベンチマークの効用水準
    cx0         x 財のベンチマーク消費量
    cy0         y 財のベンチマーク消費量
;
u0 = sam("pu","u") / pu0;
cx0 = - sam("px","u") / px0;
cy0 = - sam("py","u") / py0;

display u0, cx0, cy0;

parameter
    e_l         労働の賦存量
    e_k         資本の賦存量
    inc0        ベンチマーク所得
;
e_l = sam("pl","cons") / pl0;
e_k = sam("pk","cons") / pk0;
inc0 = - sam("pu","cons");

display e_l, e_k, inc0;

*       ----------------------------------------------------------------------
*       その他のパラメータ．
display "@ その他のパラメータ．";

parameter
    sig_x       EOS: 部門 x における資本・労働の EOS    / 0.5 /
    sig_y       EOS: 部門 y における資本・労働の EOS    / 0.5 /
    sig_u       EOS: 効用関数内の EOS                   / 1.0 /
;
display sig_x, sig_y;

parameter
    s_l         労働賦存量のスケールファクター  / 1.0 /
    s_k         資本賦存量のスケールファクター  / 1.0 /
;
display s_l, s_k;

*       ----------------------------------------------------------------------
*       生産性パラメータ
display "@ 生産性パラメータ";

parameters
        lambda        "x 部門の全要素生産性"
        lambda_k      "x 部門の資本投入についての生産性パラメータ"
        lambda_l      "x 部門の労働投入についての生産性パラメータ"
        lambda_k_     "x 部門の資本投入についての生産性パラメータ"
        lambda_l_     "x 部門の労働投入についての生産性パラメータ"
;
$ontext
資本投入の技術進歩 → lambda_k と lambda_k_ を同時に同じだけ変えないといけない。
労働投入の技術進歩 → lambda_l と lambda_l_ を同時に同じだけ変えないといけない。
$offtext
*       初期値はどれも1 
lambda = 1;
lambda_k = 1;
lambda_l = 1;
lambda_k_ = 1;
lambda_l_ = 1;
display lambda, lambda_k, lambda_l;

$ontext
MPSGEモデル

$offtext

$ontext
$model:ex_tech_01

*       アクティビティー変数 (sector) の宣言
$sectors:
    x           ! x 部門の生産量
    y           ! y 部門の生産量
    u           ! u 部門の生産量 (効用水準)

*       価格変数 (commodity) の宣言
$commodities:
    px          ! x 財の価格
    py          ! y 財の価格
    pu          ! u 財の価格 (効用の価格)
    pk          ! 労働の価格
    pl          ! 資本の価格

*       所得変数 (consumer) の宣言
$consumers:
    cons        ! 消費者の所得

*       ----------------------------------------------------------------------
*       以下，部門別の生産関数の定義

*       部門 x の生産関数
$prod:x  s:sig_x
    o:px        q:(x0*lambda)
    i:pk        q:(kx0/lambda_k)  p:(pk0*lambda_k_)
    i:pl        q:(lx0/lambda_l)  p:(pl0*lambda_l_)

*       部門 y の生産関数
$prod:y  s:sig_y
    o:py        q:y0
    i:pk        q:ky0  p:pk0
    i:pl        q:ly0  p:pl0

*       部門 u の生産関数 (効用関数)
$prod:u  s:sig_u
    o:pu        q:u0
    i:px        q:cx0  p:px0
    i:py        q:cy0  p:py0

*       ----------------------------------------------------------------------
*       予算制約
$demand:cons
    d:pu
    e:pl        q:(e_l*s_l)
    e:pk        q:(e_k*s_k)


$report:
    v:l_x       o:px    prod:x  ! x 部門の生産量
    v:l_y       o:py    prod:y  ! y 部門の生産量
    v:l_u       o:pu    prod:u  ! u 部門の生産量
    v:l_x_k     i:pk    prod:x  ! x 部門の資本の投入量
    v:l_x_l     i:pl    prod:x  ! x 部門の労働の投入量

$offtext

$sysinclude mpsgeset ex_tech_01

*       結果表示用のパラメータ
parameter
    chk_results
;

*       結果表示用のマクロの定義
$macro export_results(case) \
    chk_results("u",case) = l_u.l; \
    chk_results("x",case) = l_x.l; \
    chk_results("y",case) = l_y.l; \
    chk_results("x_k",case) = l_x_k.l; \
    chk_results("x_l",case) = l_x_l.l; \
    chk_results("px",case) = px.l / pu.l; \
    chk_results("py",case) = py.l / pu.l; \
    chk_results("pk",case) = pk.l / pu.l; \
    chk_results("pl",case) = pl.l / pu.l;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";
$ontext
Benchmark data が 均衡となっているかをチェックする．
$offtext

ex_tech_01.iterlim = 0;
$include ex_tech_01.gen
solve ex_tech_01 using mcp;

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

ex_tech_01.iterlim = 10000;
$include ex_tech_01.gen
solve ex_tech_01 using mcp;

*       "b" はベンチマークの値を表す．
export_results("b");

*       ----------------------------------------------------------------------
display "@ s1: x 部門における10%のヒックス中立的な技術進歩の効果";

lambda = 1.1;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda = 1;

export_results("s1")

*       ----------------------------------------------------------------------
display "@ s2: x 部門における10%のハロッド中立的な技術進歩の効果";

lambda_l = 1.1;
lambda_l_ = 1.1;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda_l = 1;
lambda_l_ = 1;

export_results("s2")

*       ----------------------------------------------------------------------
display "@ s3: x 部門における10%のソロー中立的な技術進歩の効果";

lambda_k = 1.1;
lambda_k_ = 1.1;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda_k = 1;
lambda_k_ = 1;

export_results("s3")

*       ----------------------------------------------------------------------
display "@ s4: s2 + s3";
$ontext
これは「10%のハロッド中立的技術進歩」+「10%のソロー中立的技術進歩」のこと
→ これは「10%のヒックス中立的技術進歩」と同じ効果のはず。
$offtext

lambda_l = 1.1;
lambda_k = 1.1;
lambda_l_ = 1.1;
lambda_k_ = 1.1;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda_l = 1;
lambda_k = 1;
lambda_l_ = 1;
lambda_k_ = 1;

export_results("s4")

*       ----------------------------------------------------------------------
display "@ s5: 「10%のハロッド中立的技術進歩」+「20%のソロー中立的技術進歩」";
lambda_l = 1.1;
lambda_k = 1.2;
lambda_l_ = 1.1;
lambda_k_ = 1.2;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda_l = 1;
lambda_k = 1;
lambda_l_ = 1;
lambda_k_ = 1;

export_results("s5")

*       ----------------------------------------------------------------------
display "@ s6: S2 + 参照価格を変えない場合";
$ontext
lambda_i は参照数量だけではなく、参照価格にも入れないといけないが、これは参照価
格に入れない場合。lamda_i 変化の効果を正確には捉えられない。

s2 のケースと結果が異なることを確認。
$offtext
lambda_l = 1.1;
* lambda_l_ = 1.1;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda_l = 1;
lambda_l_ = 1;

export_results("s6")

*       ----------------------------------------------------------------------
display "@ s7: S5 + 参照価格を変えない場合";
$ontext
参照価格にも入れないといけないが、これは参照価格に入れない場合。lamda_i 変化の効
果を正確には捉えられない。

s5 のケースと結果が異なることを確認。
$offtext
lambda_l = 1.1;
lambda_k = 1.2;
* lambda_l_ = 1.1;
* lambda_k_ = 1.2;

$include ex_tech_01.gen
solve ex_tech_01 using mcp;

lambda_l = 1;
lambda_l_ = 1;
lambda_k = 1;
lambda_k_ = 1;

export_results("s7")


*       まとめて結果を表
display chk_results;


* --------------------
* Local Variables:
* mode: gams
* fill-column: 80
* coding: utf-8
* End:
